/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package   parka;

import   java.awt.*;

class BrowserWin  extends  Frame
{
    private  ParkaNet   pn;   /* to interact with the server */   
    private  QueryWin   querywin;

    private  boolean    isNewKB;
    private  String     selectedPred;
    private  boolean    frameLabels;
    private  Thread     runFillLists;

    private  List       predList;
    private  List       inList;
    private  List       curritemList;
    private  List       outList;
    //index of values of last item selected in each list
    private  int        inSelect;
    private  int        centerSelect;
    private  int        outSelect;

    private  Panel      listPanel;
    private  Label  inLabel;
    private  Label  currLabel;
    private  Label  outLabel;

    private  TextArea   msgs;
    private  TextField  kbDomain;
    private  TextField  kbRange;

    private  Button   clear1;
    private  Button   clear2;
    private  Button   clear3;
    private  Button   exec;

    private  Menu     fileMenu;
    private  Menu     browseMenu;
    private  MenuBar  mBar;

    private  static  String       KBDOM_LABEL  = "   Domain name:";
    private  static  String       KBRAN_LABEL  = "    Range name:";
    private  static  int          KBITEM_SIZE   = 40;
    private  static  String       CLEAR1_LABEL = "  Clear In List  ";
    private  static  String       CLEAR2_LABEL = "Clear Center List";
    private  static  String       CLEAR3_LABEL = " Clear Out List  ";
    private  static  String       EXEC_LABEL   = "     Execute     ";
    private  static  String       EMPTY_LABEL  = "                 ";
    private  static  final String FILE_MENU_NAME = "File";
    private  static  final String BROWSE_MENU_NAME = "Browser";

    private  static  String[]   fileMenuItems = {"Close", "Query Window",
                                       "Application Window" };
    private  static  String[]   browseMenuItems = {"Get KB Predicates",
                                    "Shift Lists Right", "Shift Lists Left"};
    private  static  int[]  fileMenuSeps = {1};
    private  static  int[]  browseMenuSeps = null;

    private  static  boolean  ALLOW_SIMULTANEOUSLY_CHOSEN = false;
    private  static  int      NUM_VIS_ROWS = 10;
    //amt extra to give for resizing
    private  static  int      LABEL_HT_PROPORTION = 0;
    private  static  int      LIST_HT_PROPORTION = 100;

    private  static  final int  TA_NUMROWS = 2;
    private  static  final int  TA_NUMCOLS = 60;

    private  static  int      VERT_LAY_GAP = 5;
    private  static  int      HOR_LAY_GAP = 10;

    private  static  String   PREDLIST_LABEL = "KB Predicates";
    private  static  String   LINKDOM_LABEL = "Domains for Link";
    private  static  String   LINKCURR_LABEL = "Current Link";
    private  static  String   LINKRAN_LABEL = "Ranges for Link";
    private  static  String   FRAMEDOM_LABEL = "In to Frame";
    private  static  String   FRAMECURR_LABEL = "Current Frame";
    private  static  String   FRAMERAN_LABEL = "Out from Frame";
    private  static  final int UNKLIST = 0;
    private  static  final int PLIST = 1;  //predicates in KB
    private  static  final int DLIST = 2;  //in/domain
    private  static  final int CLIST = 3;  //current
    private  static  final int RLIST = 4;  //out/range
    private  static  final int UNKCHOICES = 0; //no selection, or unrecognized.
    private  static  final int LIST1 = 1;  //selection in first list only
    private  static  final int LIST2 = 2;  //selection in second list only
    private  static  final int LIST3 = 3;  //selection in third list only
    private  static  final int LIST12 = 4;  //selections in first & second list
    private  static  final int LIST23 = 5;  //selections in second & third list
    private  static  final int LIST13 = 6;  //selections in first & third list
    private  static  final int LIST123 = 7; //selections in first, second,
                                //  & third list

    private  static  final String  EMPTYPREDS = " No links exist";
    private  static  final String  MOVEPREDMSG =
          "This only works when the third list has links in it, not frames.";
    private  static  final String  MOVEFRAMEMSG =
          "This only works when the first and third lists have frames in them, not links.";
    private  static  final String  NONADJACENTCOLSEL =
             " Cannot get result from selections in non-adjacent columns";

    // start - the following constants must match values that the server
    // will understand for the command: JBROWSERQUERY
    //
	// query types
    static  final int   PREDCONST  =  1;
    static  final int   FRAMECONST =  2;
	// data column type (for returned data)
    private  static  final  String  RANGE   = "range";
    private  static  final  String  DOMAIN  = "domain";
    private  static  final  String  PREDIN  = "predin";
    private  static  final  String  PREDOUT = "predout";
    private  static  final  String  NOCOL = "nocolumn"; //used with T/F answers
    // end -- constants agreed on with the parka server

    BrowserWin (ParkaNet pn, QueryWin qwin)
    {
	this.pn = pn;
        querywin = qwin;

	isNewKB = true;   //ready to get data for a new KB.
	selectedPred = null;
	frameLabels = false;   //When true: 2nd and 4th lists have links;
                               //3rd list is frames;  opposite when false.
	runFillLists = null;
	inSelect = -1;
	centerSelect = -1;
	outSelect = -1;

	setTitle("Parka Browser");

	        // Add menus to window
        MenuMaker   makeMenu;

        makeMenu = new MenuMaker (FILE_MENU_NAME, MenuMaker.SIMPLE,
                           fileMenuItems, fileMenuSeps);
        fileMenu = makeMenu.createMenu();

	//NOTE:  Change index/file ref if move menu item for raising the
	//       application window.
	//
	if (ApplType.getIsApplet())
        {          // make this menu item insensitive:
            MenuItem  mi = fileMenu.getItem (3);
	    mi.disable();
        }

        makeMenu = new MenuMaker (BROWSE_MENU_NAME, MenuMaker.SIMPLE,
                           browseMenuItems, browseMenuSeps);
        browseMenu = makeMenu.createMenu();

        mBar = new MenuBar();
        mBar.add (fileMenu);
        mBar.add (browseMenu);
        setMenuBar(mBar);

        setLayout (new BorderLayout (HOR_LAY_GAP, VERT_LAY_GAP));

        // Text area for messages
	msgs = new TextArea ("", TA_NUMROWS, TA_NUMCOLS);
	msgs.setEditable (false);
	add ("South", msgs);

	// Add lists
        listPanel = makeListPanel();
	add ("Center", listPanel);

	// Add buttons and text field for entry of KB constants
	Panel q = makeChoicePanel();
	add ("North", q);

        pack();
        validate();
    }

    private Panel makeChoicePanel()
    {
	Panel  pan = new Panel();

	// Choose layout style
	GridBagLayout  gridBag = new GridBagLayout();
	pan.setLayout (gridBag);
	GridBagConstraints gbc = new GridBagConstraints();

	// Make the components
	Label frameLabel = new Label (KBDOM_LABEL, Label.LEFT);
	Label linkLabel = new Label (KBRAN_LABEL, Label.LEFT);
	kbDomain = new TextField ("", KBITEM_SIZE);
	kbRange = new TextField ("", KBITEM_SIZE);
	clear1 = new Button (CLEAR1_LABEL);
	clear2 = new Button (CLEAR2_LABEL);
	clear3 = new Button (CLEAR3_LABEL);
	exec = new Button (EXEC_LABEL);

	setGridBagConstraints (gbc, 0, 0, 1, 1, 20, 48,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (frameLabel, gbc);
	pan.add (frameLabel);

	setGridBagConstraints (gbc, 1, 0, 2, 1, 0, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (kbDomain, gbc);
	pan.add (kbDomain);

	setGridBagConstraints (gbc, 3, 0, 1, 1, 0, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (linkLabel, gbc);
	pan.add (linkLabel);

	setGridBagConstraints (gbc, 4, 0, 2, 1, 0, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (kbRange, gbc);
	pan.add (kbRange);

	setGridBagConstraints (gbc, 1, 1, 1, 1, 20, 48,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (clear1, gbc);
	pan.add (clear1);

	setGridBagConstraints (gbc, 2, 1, 1, 1, 20, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (clear2, gbc);
	pan.add (clear2);

	setGridBagConstraints (gbc, 3, 1, 1, 1, 20, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (clear3, gbc);
	pan.add (clear3);

	setGridBagConstraints (gbc, 4, 1, 1, 1, 20, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (exec, gbc);
	pan.add (exec);

	Label fake_for_layout = new Label (EMPTY_LABEL);
	setGridBagConstraints (gbc, 5, 1, 1, 1, 20, 0,
                           GridBagConstraints.NONE, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (fake_for_layout, gbc);
	pan.add (fake_for_layout);

	return  pan;
    }

    private Panel makeListPanel()
    {
	Panel  pan = new Panel();

	// Choose layout style
	GridBagLayout  gridBag = new GridBagLayout();
	pan.setLayout (gridBag);
	GridBagConstraints gbc = new GridBagConstraints();

	// Make labels for each list
	Label  predLabel = new Label (PREDLIST_LABEL, Label.CENTER);
	inLabel = new Label (LINKDOM_LABEL, Label.CENTER);
	currLabel = new Label (LINKCURR_LABEL, Label.CENTER);
	outLabel = new Label (LINKRAN_LABEL, Label.CENTER);

	// Add the lists
	// Note about lists:  Lists add scrollbars on an "As Needed"
	// basis, not under programmer control.  If the list is longer
	// than the space available in the current size window, scrollbars
	// will automatically be added.  When the list is shorter, they
	// automatically disappear.
	//
	predList = new List (NUM_VIS_ROWS, ALLOW_SIMULTANEOUSLY_CHOSEN);
	inList = new List (NUM_VIS_ROWS, ALLOW_SIMULTANEOUSLY_CHOSEN);
	curritemList = new List (NUM_VIS_ROWS, ALLOW_SIMULTANEOUSLY_CHOSEN);
	outList = new List (NUM_VIS_ROWS, ALLOW_SIMULTANEOUSLY_CHOSEN);

	setGridBagConstraints (gbc, 0, 0, 1, 1, 25, LABEL_HT_PROPORTION,
                     GridBagConstraints.HORIZONTAL, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (predLabel, gbc);
	pan.add (predLabel);
	setGridBagConstraints (gbc, 1, 0, 1, 1, 25, LABEL_HT_PROPORTION,
                     GridBagConstraints.HORIZONTAL, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (inLabel, gbc);
	pan.add (inLabel);
	setGridBagConstraints (gbc, 2, 0, 1, 1, 25, LABEL_HT_PROPORTION,
                     GridBagConstraints.HORIZONTAL, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (currLabel, gbc);
	pan.add (currLabel);
	setGridBagConstraints (gbc, 3, 0, 1, 1, 25, LABEL_HT_PROPORTION,
                     GridBagConstraints.HORIZONTAL, HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (outLabel, gbc);
	pan.add (outLabel);

	setGridBagConstraints (gbc, 0, 1, 1, GridBagConstraints.REMAINDER,
                     0, LIST_HT_PROPORTION, GridBagConstraints.BOTH,
                     HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (predList, gbc);
	pan.add (predList);
	setGridBagConstraints (gbc, 1, 1, 1, GridBagConstraints.REMAINDER,
                     0, LIST_HT_PROPORTION, GridBagConstraints.BOTH,
                     HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (inList, gbc);
	pan.add (inList);
	setGridBagConstraints (gbc, 2, 1, 1, GridBagConstraints.REMAINDER,
                     0, LIST_HT_PROPORTION, GridBagConstraints.BOTH,
                     HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (curritemList, gbc);
	pan.add (curritemList);
	setGridBagConstraints (gbc, 3, 1, 1, GridBagConstraints.REMAINDER,
                     0, LIST_HT_PROPORTION, GridBagConstraints.BOTH,
                     HOR_LAY_GAP, VERT_LAY_GAP);
	gridBag.setConstraints (outList, gbc);
	pan.add (outList);

	return  pan;
    }

    void setGridBagConstraints (GridBagConstraints constraints,
              int gx, int gy, int gwid, int ght, int wtx, int wty,
              int fill, int xinset, int yinset)
    {
	constraints.gridx = gx;
	constraints.gridy = gy;
	constraints.gridwidth = gwid;
	constraints.gridheight = ght;
	constraints.weightx = wtx;
	constraints.weighty = wty;
	constraints.fill = fill;
	constraints.insets = new Insets (yinset, xinset, yinset, xinset);
    }

    public Dimension minimumSize()
    {
	Dimension  minSize = super.minimumSize();
	int        minHt = Math.max (150, minSize.height);
	return  (new Dimension (minSize.width, minHt));
    }

    public Dimension preferredSize()
    {
	Dimension  prefSize = super.preferredSize();
	int        prefHt = Math.max (250, prefSize.height);
	return  (new Dimension (prefSize.width, prefHt));
    }

    // whenever an old KB is successfully exited, this routine should
    // be called, since the connection no longer exists from ParkaNet
    // for browsing that KB.
    //
    void oldKBclosed()
    {
	isNewKB = true;
	selectedPred = null;
	initLists();
	initListLabels();
	pn.initBrowserData();
	clearDisplayForBrowser();
	kbDomain.setText("");
	kbRange.setText("");
    }

    void  initLists()
    {
	predList.clear();
	inList.clear();
	curritemList.clear();
	outList.clear();
	inSelect = -1;
	centerSelect = -1;
	outSelect = -1;
    }

    void  initListLabels()
    {
	frameLabels = false;
        setLinkLabels();
    }

    // When the "as needed" scrollbars add, the list to the right
    // is obscured at its left edge.  Nothing seems to counteract this;
    // not even our attempts to resize the window within the program.
    // I think its a timing problem;  the resize has to happen after
    // the scrollbar additions, and our attempts here are not doing that.
    // XXX TO DO: Perhaps we should start a thread that sleeps awhile, then
    // resizes??  Kind of kludgy, but can't think of another way.
    //
    void validate_container()
    {
        predList.invalidate();
        inList.invalidate();
        curritemList.invalidate();
        outList.invalidate();
	validate();
    }

    void displayForBrowser (String  message)
    {
	String currText = msgs.getText();
	if (currText.length() > 0)
	    msgs.setText (currText + "  " + message);
	else
	    msgs.setText (message);
    }

    void clearDisplayForBrowser()
    {
	msgs.setText ("");
    }

    void setFrameLabels()
    {
	inLabel.setText (FRAMEDOM_LABEL);
	currLabel.setText (FRAMECURR_LABEL);
	outLabel.setText (FRAMERAN_LABEL);
    }

    void setLinkLabels()
    {
	inLabel.setText (LINKDOM_LABEL);
	currLabel.setText (LINKCURR_LABEL);
	outLabel.setText (LINKRAN_LABEL);
    }

    void setListLabels()
    {
	if (frameLabels)
	    setLinkLabels();
	else
	    setFrameLabels();
	frameLabels = !frameLabels;
    }

    void clearInList()
    {
	// some peers complain if 'clear()' is called on an empty list:
	if (inList.countItems() > 0)
	    inList.clear();
	inSelect = -1;
    }

    void clearOutList()
    {
	// some peers complain if 'clear()' is called on an empty list:
	if (outList.countItems() > 0)
	    outList.clear();
	outSelect = -1;
    }

    void clearCenterList()
    {
	// some peers complain if 'clear()' is called on an empty list:
	if (curritemList.countItems() > 0)
	    curritemList.clear();
	centerSelect = -1;
    }

    private List getList (String colName)
    {
	List   l;

	if (colName.equals (RANGE))
	    l = outList;
	else   if (colName.equals (DOMAIN))
	    l = inList;
	else   if (colName.equals (PREDIN))
	    l = inList;
	else   if (colName.equals (PREDOUT))
	    l = outList;
	else
	{
	    System.out.println ("BrowserWin.getList() -- Unrecognized " +
                         "column name: <" + colName + ">");
	    l = null;
	}
	return  l;
    }

    void fillList (String[] column)
    {
	List   currentList = null;

        for (int j = 0; j < column.length; j++)
        {
            String   entry = column[j];
            if (j == 0)    //need to determine which col we have
            {
		if (column.length > 1)
		{
		    if (entry.equals (NOCOL))
		    {
			displayForBrowser (column[j+1]);
			break;
		    }
                    else
                    {
                        currentList = getList (entry);
                        if (currentList == null)
                            break;  //error message given in "getList()"
                    }
		}
		else  // column.length == 1
		    displayForBrowser (" Column that would contain " +
                          entry + " is empty");
            }
            else      //actual col entry
            {
                currentList.addItem (entry);
            }
        }
        if ((currentList == null) && (column.length > 1) &&
                               !(column[0]).equals(NOCOL))
	    validate_container();
    }

    void putSelectedPredicate()
    {
	selectedPred = predList.getSelectedItem();
	curritemList.addItem (selectedPred);
	int currlistSize = curritemList.countItems();
	if (currlistSize > 0)
	    curritemList.makeVisible (currlistSize -1); //last item
    }

    //delselect 'from' and select 'to'
    void resetSelections (List from, List to, int newIndex)
    {
	if (from == inList)
	    inSelect = -1;
	else   if (from == curritemList)
	    centerSelect = -1;
	else   if (from == outList)
	    outSelect = -1;

	if (to == inList)
            inSelect = newIndex;
	else   if (to == curritemList)
            centerSelect = newIndex;
        else   if (to == outList)
            outSelect = newIndex;
    }

    void moveContents (List from, List to)
    {
	to.clear();
	int  selitem = from.getSelectedIndex(); //only one at a time

	int  numitems = from.countItems();
	for (int i = 0; i < numitems; i++)
	{
	    String  item = from.getItem (i);
	    to.addItem (item, i);
	}

	if (selitem != -1)
	{
	    to.select (selitem); //keep same item selected
	    to.makeVisible (selitem);
	    validate_container();
	}
	resetSelections (from, to, selitem);

	from.clear();
    }

    static String getColumnName (String colType)
    {
	String  colName;

	if (colType.equals (RANGE))
	    colName = new String("'" + LINKRAN_LABEL + "'");
	else   if (colType.equals (DOMAIN))
	    colName = new String("'" + LINKDOM_LABEL + "'");
	else   if (colType.equals (PREDIN))
	    colName = new String("'" + FRAMEDOM_LABEL + "'");
	else   if (colType.equals (PREDOUT))
	    colName = new String("'" + FRAMERAN_LABEL + "'");
	else
	{
	    System.out.println ("Programmer Error: column type is " +
                  colType + ". No column expected to match.");
	    colName = new String("");
	}

	return  colName;
    } 

    void getFramesForPred (String chosenPred)
    {
/* TEST
	System.out.println ("Pred Clicked on " + chosenPred);
*/
	if (runFillLists != null)
	    joinFillThread();

        //the called class will use a thread; but it should not
	//interfere with other threads, since we could not have gotten
	//here if a KB wasn't loaded and predicates listed in the
	//another browser list.
	//
	PredicateData pdata = new PredicateData (chosenPred, pn, this);
	runFillLists = new Thread (pdata, "Browser PredGetter");
	runFillLists.start();
    }

    void getPredsForFrame (String chosenFrame)
    {
/* TEST
	System.out.println ("Frame Clicked on " + chosenFrame);
*/
	if (runFillLists != null)
	    joinFillThread();

	FrameData fdata = new FrameData (chosenFrame, pn, this,
                                 FrameData.PRED_UNRESTRICT);
	runFillLists = new Thread (fdata, "Browser FrameGetter");
	runFillLists.start();
    }

    /* decided to not use currently; see commented out call */
    private void getUnrestrictedPredout (String frame)
    {
/* TEST
	System.out.println ("getUnrestrictedPredout():  Frame == " + frame);
*/
	// message, because can only get here if a list 2 pred was also selected
	displayForBrowser (
              "Predicate in list 2 did not affect outcome in list 4");
	inList.deselect(inList.getSelectedIndex());

	if (runFillLists != null)
	    joinFillThread();

	FrameData fdata = new FrameData (frame, pn, this,
                                 FrameData.PRED_UNRESTRICT_OUT);
	runFillLists = new Thread (fdata, "Browser FrameGetter");
	runFillLists.start();
    }

    /* decided to not use currently; see commented out call */
    private void getUnrestrictedPredin (String frame)
    {
	// message, because can only get here if a list 4 pred was also selected
	displayForBrowser (
              "Predicate in list 4 did not affect outcome in list 2");
	outList.deselect(outList.getSelectedIndex());
/* TEST
	System.out.println ("getUnrestrictedPredin():  Frame == " + frame);
*/
	if (runFillLists != null)
	    joinFillThread();

	FrameData fdata = new FrameData (frame, pn, this,
                                 FrameData.PRED_UNRESTRICT_IN);
	runFillLists = new Thread (fdata, "Browser FrameGetter");
	runFillLists.start();
    }

    private void getRestrictedDomain (String pred, String range)
    {
/* TEST
	System.out.println ("getRestrictedDomain():  " + pred + " " + range);
*/
	if (runFillLists != null)
	    joinFillThread();

	PredicateData pdata = new PredicateData (pred, pn, this);
	pdata.setRangeValue (range);
	runFillLists = new Thread (pdata, "Browser PredGetter");
	runFillLists.start();
    }

    private void getRestrictedRange (String domain, String pred)
    {
/* TEST
	System.out.println ("getRestrictedRange():  " + domain + " " + pred);
*/
	if (runFillLists != null)
	    joinFillThread();

	PredicateData pdata = new PredicateData (pred, pn, this);
	pdata.setDomainValue (domain);
	runFillLists = new Thread (pdata, "Browser PredGetter");
	runFillLists.start();
    }

    void verify (String dom, String pred, String ran)
    {
/* TEST
	System.out.println ("verify():  " + dom + " " + pred + " " + ran);
*/
	if (runFillLists != null)
	    joinFillThread();

	PredicateData pdata = new PredicateData (pred, pn, this);
	pdata.setDomainValue (dom);
	pdata.setRangeValue (ran);
	runFillLists = new Thread (pdata, "Browser PredGetter");
	runFillLists.start();
    }

    // fill the inList and outList with data from the requested predicate
    //
    void loadFrameLists (String[] domrangPredData)
    {
	int  listLength = domrangPredData.length/2;
	for (int i = 0, j = 0; i < listLength; i++)
	{
	    inList.addItem (domrangPredData[j]);      j++;
	    outList.addItem (domrangPredData[j]);     j++;
	}
	validate_container();
    }

    int getList (List thislist)
    {
	if (thislist == predList)
	    return  PLIST;
	else   if (thislist == inList)
	    return  DLIST;
	else   if (thislist == curritemList)
	    return  CLIST;
	else   if (thislist == outList)
	    return  RLIST;
	else
	{
	    System.out.println ("Programmer Error:  No such list in browser");
	    return  UNKLIST;
	}
    }

    private void joinFillThread()
    {
	if (runFillLists != null)
	{
	    try   {
	        runFillLists.join();
	    }
	    catch  (InterruptedException  ie)  {
	        System.out.println (runFillLists.getName() +
                         " thread was interrupted.");
	    }
	    runFillLists = null;
	}
    }

    public boolean handleEvent(Event evt)
    {
        if (evt.id == Event.WINDOW_DESTROY)
        {
	    joinFillThread();
	    querywin.closeBrowserWin();
            return true;
        }
	else if (evt.target instanceof List)
	{
	    List  currlist = (List)evt.target;

            if (evt.id == Event.LIST_SELECT)
	    {
		clearDisplayForBrowser();

		// do this in here, so not executed for every event:
	        int    whichList = getList (currlist);
		int    currSel = currlist.getSelectedIndex();

	        joinFillThread();

		switch (whichList)
		{
	          case PLIST: if (!frameLabels)
                                  putSelectedPredicate();
			      else
			          displayForBrowser(MOVEPREDMSG);
			      break;
                  //Next three lists can have only one selected item at a
                  //time.  If click on a new item, selection switches to
                  //new item.  If click on already selected item, you 
                  //unselect it.
                  //
	  	  case DLIST: if (currSel == inSelect)
                              {   //now no selection exists
	                          inList.deselect (inSelect);
				  inSelect = -1;
                              }
                              else    //curr selection is now old
			          inSelect = currSel;
			      break;
	  	  case CLIST: if (currSel == centerSelect)
                              {   //now no selection exists
                                  curritemList.deselect (
                                           curritemList.getSelectedIndex());
                                  centerSelect = -1;
                              }
                              else    //curr selection is now old
                                  centerSelect = currSel;
		 	      break;
	  	  case RLIST: if (currSel == outSelect)
                              {   //now no selection exists
	                          outList.deselect (outList.getSelectedIndex());
				  outSelect = -1;
                              }
                              else    //curr selection is now old
			          outSelect = currSel;
			      break;
	  	  case UNKLIST:
			System.out.println ("Unrecognized list in browser");
		}
/* TEST
		System.out.println ("inSelect == " + inSelect +
                          " ; centerSelect == " + centerSelect +
                                " ; outSelect == " + outSelect);
*/
                return true;
	    }
	    // NOTE:  The application version of the browser does not ever
            // invoke the LIST_DESELECT event;  selection of a selected item
	    // is still the LIST_SELECT event.  But the applet version does.
	    // We are handling both kinds of invocation, so we include
	    // anything that involves the deselect option here and above
	    // in LIST_SELECT event processing.
	    //
            else  if (evt.id == Event.LIST_DESELECT)
	    {
		clearDisplayForBrowser();

		// do this in here, so not executed for every event:
	        int    whichList = getList (currlist);

	        joinFillThread();

		switch (whichList)
		{
	          case PLIST: break;
                  //Next three lists can have only one selected item at a
                  //time or no selected item is ok.
                  //
	  	  case DLIST: inSelect = -1;
			      break;
	  	  case CLIST: centerSelect = -1;
		 	      break;
	  	  case RLIST: outSelect = -1;
			      break;
	  	  case UNKLIST:
			System.out.println ("Unrecognized list in browser");
		}
/* TEST
		System.out.println ("inSelect == " + inSelect +
                         " ; centerSelect == " + centerSelect +
                                " ; outSelect == " + outSelect);
*/
                return true;
	    }
	}

        return super.handleEvent(evt);
    }

    public boolean keyDown (Event evt, int key)
    {
	String   domain;
	String   range;
	boolean  handled = false;

	if ((evt.target instanceof TextField) &&
               (evt.id == Event.KEY_PRESS))
	{
	    TextField tf = (TextField)evt.target;

	    if (evt.key == 10)  // 10 is return key
	    {
		clearDisplayForBrowser();

		if (frameLabels)
		    displayForBrowser (MOVEFRAMEMSG);
		else
		{
		    int  listsize;

		    if (tf == kbDomain)
		    {
			domain = tf.getText();
			if (!domain.equals(""))
			{
			    inList.addItem (domain);
                            inList.makeVisible (inList.countItems() -1);
                                       //last item
			}
		    }
		    else  if (tf == kbRange)
		    {
			range = tf.getText();
			if (!range.equals(""))
			{
			    outList.addItem (range);
                            outList.makeVisible (outList.countItems() -1);
			}
		    }
		    else
		        System.out.println ("Textfield not recognized. " +
                                 "Event ignored");
		}

		handled = true;
	    }
	}
	return  handled;
    }

    private int findListAction()
    {
	int   listAxn = UNKCHOICES;

	if (centerSelect != -1)   // 1,2,3 or 1,2 or 2,3 or 2
        {
	    if (inSelect != -1)   // 1,2,3 or 1,2
		if (outSelect != -1)   // 1,2,3
		    listAxn = LIST123;
		else                          // 1,2
		    listAxn = LIST12;
	    else  if (outSelect != -1) // 2,3
		listAxn = LIST23;
	    else                       // 2
		listAxn = LIST2;
        }
        else if (inSelect != -1)  // 1 or 1,3
        {
	    if (outSelect != -1)  // 1,3
		listAxn = LIST13;
	    else                         // 1
		listAxn = LIST1;
        }
        else if (outSelect != -1) // 3
	    listAxn = LIST3;

	return   listAxn;
    }

    private void executeTraversal()
    {
	int  listAction = findListAction();

	switch (listAction)
	{
	  case LIST1:   setListLabels();  //note that this sets 'frameLabels'
			moveContents (inList, curritemList);
			clearOutList();  //clear remaining list
			if (frameLabels)
			    getPredsForFrame (
                                     curritemList.getItem(centerSelect));
			else
			    getFramesForPred (
                                     curritemList.getItem(centerSelect));
			break;

	  case LIST2:   clearInList();
			clearOutList();
			if (frameLabels)
			    getPredsForFrame (
                                     curritemList.getItem(centerSelect));
			else
			    getFramesForPred (
                                     curritemList.getItem(centerSelect));
			break;

	  case LIST3:   setListLabels();
			moveContents (outList, curritemList);
			clearInList();  //clear remaining list
			if (frameLabels)
			    getPredsForFrame (
                                     curritemList.getItem(centerSelect));
			else
			    getFramesForPred (
                                     curritemList.getItem(centerSelect));
			break;

	  case LIST12:
			if (frameLabels)
			{
/*                      Switch action from this:
                            clearOutList();
			    getUnrestrictedPredout (
                                     curritemList.getItem(centerSelect));
                        To this:
*/
                            setListLabels();
                            moveContents (curritemList, outList);
                            moveContents (inList, curritemList);
			    getRestrictedDomain (
                                     curritemList.getItem(centerSelect),
				     outList.getItem(outSelect));
			}
			else
			{
                            clearOutList();
			    getRestrictedRange (inList.getItem(inSelect),
                                     curritemList.getItem(centerSelect));
			}
			break;

	  case LIST23:  if (frameLabels)
			{
/*                      Switch action from this:
                            clearInList();
			    getUnrestrictedPredin (
                                     curritemList.getItem(centerSelect));
                        To this:
*/
                            setListLabels();
                            moveContents (curritemList, inList);
                            moveContents (outList, curritemList);
			    getRestrictedRange (inList.getItem(inSelect),
                                     curritemList.getItem(centerSelect));
			}
			else
			{
                            clearInList();
			    getRestrictedDomain (
                                     curritemList.getItem(centerSelect),
				     outList.getItem(outSelect));
			}
			break;

	  case LIST13:  displayForBrowser (NONADJACENTCOLSEL);
			break;

	  case LIST123: if (!frameLabels)
                            verify (inList.getItem(inSelect),
                                   curritemList.getItem(centerSelect),
                                       outList.getItem(outSelect));
                        else
			    displayForBrowser (" Ambiguous; the 2 predicates are unrelated to each other");
			break;

	  case UNKCHOICES:
			System.out.println ("No highlighted choices to execute");
	}
    }

    public boolean action (Event evt, Object arg)
    {
	boolean   result = true;   //assume handled

	if (evt.target instanceof Button)
	{
	    String  buttonLabel = (String)arg;

	    joinFillThread();

	    if (buttonLabel.equals (EXEC_LABEL))
	    {
		executeTraversal();
	    }
	    else  if (buttonLabel.equals (CLEAR1_LABEL))
	    {
		clearDisplayForBrowser();
		clearInList();
	    }
	    else  if (buttonLabel.equals (CLEAR2_LABEL))
	    {
		clearDisplayForBrowser();
		clearCenterList();
	    }
	    else  if (buttonLabel.equals (CLEAR3_LABEL))
	    {
		clearDisplayForBrowser();
		clearOutList();
	    }
	    else
		System.out.println ("Unrecognized button in browser");
	}
	else  if (evt.target instanceof MenuItem)
	{
	    String label = (String) arg;  //menu label

	    if (label.equals (fileMenuItems[0]))
	    {
		querywin.closeBrowserWin();
	    }
	    else  if (label.equals (fileMenuItems[1]))
	    {
		querywin.toFront();
	    }
	    else  if (label.equals (fileMenuItems[2]))
	    {
		querywin.appWinToFront();
	    }
	    else   if (label.equals (browseMenuItems[0]))
	    {
		clearDisplayForBrowser();
		createPredList();
	    }
	    else   if (label.equals (browseMenuItems[1]))
	    { // "Shift lists right"
		clearDisplayForBrowser();
		moveContents (curritemList, outList);
		moveContents (inList, curritemList);
		setListLabels();
	    }
	    else   if (label.equals (browseMenuItems[2]))
	    { // "Shift lists left"
		clearDisplayForBrowser();
		moveContents (curritemList, inList);
		moveContents (outList, curritemList);
		setListLabels();
	    }
	    else
		System.out.println ("Unrecognized menu item in browser");
	}
	else
	    result = false;

	return   result;
    }

    private void createPredList()
    {
	if (isNewKB)
	{
	    String[]  predlist = pn.getPredicateList();
	    if (predlist != null)
	    {
		if (predList.countItems() > 0)
		    predList.clear();

	        for (int i = 0; i < predlist.length; i++)
		    predList.addItem (predlist[i]);
	        isNewKB = false;
	    }
	    else
	    {
		 displayForBrowser (EMPTYPREDS);
	    }
	}
	// else the predicates we have are valid
    }
}
