/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
/**********************************************************************
  drawing area
*********************************************************************/
package parka;

import java.awt.*;
import java.util.*;

public class DrawArea extends java.awt.Panel
{
    // size of virtual screen
    final static int MAX_SIZE = 1000;
    // background color
    final static Color BG_COLOR = Color.white;

    //default location for creation of new Icon:
    final static int  DEFAULT_NEWICON_X = 30;
    final static int  DEFAULT_NEWICON_Y = 30;

    int     offset_x, offset_y;    //scrollbar values
    private Scrollbar hbar, vbar;

    private QueryWin parent;
    private ToolBar  tb;

    private Image oi;
    private Graphics og;

    // GraphObjects
    Vector icons,     //represents frames
           links;     //represents predicates

    GraphObject active,          // the currently active icon or link
                rcpt = null;

    // Predicate Action constants
    private static final int UNSET = 0;
    private static final int NEWLINK = 1;
    private static final int RENAMELINK = 2;
    private  int  predAction;

    // Font information
    Font f;
    FontMetrics fm;

    // *******
    // Methods
    // *******

    public DrawArea (QueryWin MyWin, ToolBar tb)
    {
        //implicit super() call here creates the panel
        
        // icon and link vectors
        icons = new Vector();
        links = new Vector();

        // scrolling setup
        hbar = create_scrollbar (Scrollbar.HORIZONTAL, 0, 0, 0, MAX_SIZE-1);
        vbar = create_scrollbar (Scrollbar.VERTICAL, 0, 0, 0, MAX_SIZE-1);
        offset_x = 0;
        offset_y = 0;
	predAction = UNSET;  //undefined
	active = null;

        // set up double buffering
        //    oi = createImage(1,1);
        //    og = oi.getGraphics();

        // remember your window
        parent = MyWin;
        // and your associated toolbar
        this.tb = tb;

        // Font setup
        f = new Font(parent.fontname, Font.PLAIN, parent.fontsize);
        fm = parent.parent.getFontMetrics(f);

        // Layout
        setLayout(new BorderLayout());
        add ("South", hbar);
        add ("East", vbar);

        // get all events
        requestFocus();
    }

    Scrollbar create_scrollbar (int orient, int value, int vis,
                                     int min, int max)
    {
	Scrollbar  b = new Scrollbar (orient, value, vis, min, max);
	/* value = init value of sb; vis (visibility) = visible part of
           component "below"; min sb value; max sb value   */
	b.setLineIncrement (MAX_SIZE/100);  /* unit increment of sb */
	b.setPageIncrement (MAX_SIZE/10);  /* block/page increment of sb */

	return  b;
    }

    //'paint()' is directly called rather than repaint() because it
    //gives extremely good effects, like double buffering, without the
    //overhead.  Also, the temporary line drawn for the link remains
    //on the screen; a straightforward change to repaint calls where paint
    //calls now exist gives a temp link line that disappears when there are
    //no more mouse drag events.
    //
    public void paint (Graphics g)
    {
	if (ApplType.getIsVersion1_1())
            super.paint(g);  /* Parka object tries to call Container.paint()
                 when inside netscape -- possibly because this is compiled
                 with a 1.1 compiler, so we avoid this.  Component.paint()
                 just calls this paint again, and would be an infinite loop
                 in netscape; so there is no else clause. */

        // erase previous image
        og.setColor(BG_COLOR);
        og.fillRect(0,0,bounds().width, bounds().height);

        drawGraphObjects (og);
        g.drawImage (oi, 0, 0, this);
    }

    public void drawGraphObjects(Graphics g)
    {
        Icon icon;
        Link link;
        Enumeration e;

        // draw links
        e = links.elements();
        while (e.hasMoreElements())
        {
            try
            {
                link = (Link)e.nextElement();
                link.drawGO(g, offset_x, offset_y);
            }
            catch (NoSuchElementException e1)
            {
                // Should not get here.
		System.out.println ("Exception: No such link");
            }
        }

        // draw icons
        e = icons.elements();
        while (e.hasMoreElements())
        {
            try
            {
                icon = (Icon)e.nextElement();
                icon.drawGO(g, offset_x, offset_y);
            }
            catch (NoSuchElementException e1)
            {
                // Should not get here.
		System.out.println ("Exception: No such frame icon");
            }
        }
    }

    //delete all graph objects
    //
    void clearGraph()
    {
	links.removeAllElements();
	icons.removeAllElements();
        selectObject(null);    //does redraw
    }

    void translate()
    {
        offset_x = hbar.getValue();
        offset_y = vbar.getValue();
        paint (getGraphics());
    }

    public synchronized void reshape(int x, int y, int width, int height)
    {
        super.reshape(x, y, width, height);
   
        Dimension d = size();
        if (d.width == 0) d.width=1;
        if (d.height == 0) d.height=1;
      
        oi = createImage(d.width, d.height);
        og = oi.getGraphics();

        hbar.setValues (hbar.getValue(), width,hbar.getMinimum(),
                                hbar.getMaximum());
        vbar.setValues (vbar.getValue(),height, vbar.getMinimum(),
                                vbar.getMaximum());
      
        paint (getGraphics());
    }

    // a debug routine
    static String printPredAction (int predAxn)
    {
	String  pa;
	switch (predAxn)
	{
	    case  UNSET:      pa = " unset ";
                              break;
	    case  NEWLINK:    pa = " new link ";
                              break;
	    case  RENAMELINK: pa = " rename link ";
                              break;
	    default:          pa = " unknown state ";
	}
	return  pa;
    }

    // either create new link, or rename existing link
    void doLinkAction (String  selectedItem)
    {
/* TEST
	System.out.println ("Entered 'doLinkAction()' with selection: " +
                          selectedItem + " and pred action: " +
                          DrawArea.printPredAction (predAction));
*/
        if (predAction == NEWLINK)
            finishLink (selectedItem);
        else if (predAction == RENAMELINK)
            active.renameGO (selectedItem);

        paint (getGraphics());
    }

    //designate domain for the link
    //
    void beginCreateLink (int x, int y)
    {
/* TEST
	System.out.println ("Entered 'beginCreateLink()'");
*/
	// find which object got hit
	GraphObject  go = hitTest(x,y);
	if (go  instanceof  Icon)
	{
/* TEST
	System.out.println ("Icon selected");
*/
	    predAction = NEWLINK;
	    selectObject(go);   //does a redraw
	}
	else
	{
	    predAction = UNSET;
	    selectObject(null); //does a redraw
	}

        requestFocus();
    }

    void askForPredicate()
    {
/* TEST
	System.out.println ("Entered 'askForPredicate()'");
*/
	String[]   predlist = parent.pn.getPredicateList();

	PredDialog p = new PredDialog (parent, this, predlist);

	if (ApplType.getIsVersion1_0())
	    ((Window)p).show();  //Netscape thinks we want Dialog.show()
                         //which doesn't exist until java 1.1
	else
	    p.show();
    }

    //both ends of link are now designated
    //
    public void createLink (int x, int y)
    {
/* TEST
	System.out.println ("Entered 'createLink()'");
*/
	if (predAction == NEWLINK)
	{
            rcpt = hitTest(x,y);

            if ((active != null) && (active instanceof Icon) &&
             (rcpt != null) && (rcpt != active) && (rcpt instanceof Icon))
	    {
                askForPredicate();
	    }
	}
    }

    // finish creation of a new link, after dialog used to get name for it.
    //
    void finishLink (String predStr)
    {
	/*  DEBUG
              System.out.println ("Inside 'finishLink()'");
              System.out.println ("     predStr is:  " + predStr);
	*/
        // create new icon, pass in this component & coordinates
        Link link = new Link (this, (Icon)active, (Icon)rcpt, predStr, f, fm);

        // add link to vector
        links.addElement(link);

        // Make the link the active object;  Redraw
        selectObject(link);
        predAction = UNSET;
    }

    public Icon createIcon(int x, int y)
    {
        // create new icon, pass in this component & coordinates
        Icon icon = new Icon(this, x, y, f, fm);

        // add icon to vector
        icons.addElement(icon);

        // Make the Icon the active object
        selectObject(icon);

	// Prepare for a rename action on current Icon
	tb.activateTextField();

//TEST -- need the 'paint()'?
	paint (getGraphics());

        return icon;
    }

    // Used to toggle Icon value when the check box changes
    public void toggleVarStatus(boolean newstate)
    {
        if ((active != null) && (active instanceof Icon))
        {
            changeVariable(newstate, false); // Infinite recursion???

//TEST:  need the 'paint()'?
            paint (getGraphics());
        }
    }

    public boolean mouseEnter (Event evt, int x, int y)
    {
        tb.resetCursor();
	    //make sure correct cursor is drawn for
	    //current tool
        return true;
    }

    public boolean mouseExit (Event evt, int x, int y)
    {
        parent.setCursor (Frame.DEFAULT_CURSOR);
        return true;
    }

    // handleEvent() handles the scrollbar events.
//TEST  -- delete or revise this comment later....
    // It updates the offset_x and offset_y variables that
    // are used by the canvas.

    //toggle the Variable item/button with a key
    //
    public boolean keyDown (Event evt, int key)
    {
        //System.out.println("ASCII is " + key);

        if (evt.id == Event.KEY_PRESS)
	{
            if ((active instanceof Icon) && ((evt.key == Event.ESCAPE) ||
                   (evt.key == Event.BACK_SPACE) || (evt.key == Event.DELETE)))
/* Java 1.0 may need the constant numbers... 
            if ((active instanceof Icon) && ((evt.key == 27) ||
                   (evt.key == 8) || (evt.key == 127)))
*/
            { // Toggle variable status with one of the the keys above
                changeVariable (false, true);
                return true;
            }
	}
	return false;
    }

    // Java only recognizes one mouse button (the left one), for compatibility
    // with systems with only one button.  Others with multiple buttons
    // still generate mouse events, but Java sees the event as the mouse 1
    // key plus various modifiers.  The mouse buttons therefore generate
    // the following in an Event record:
    //
    // Mouse 1 button (left):    a mouse button without modifiers
    // Mouse 2 button (middle):  a mouse button plus Event.ALT_MASK
    // Mouse 3 button (right):   a mouse button plus Event.META_MASK
    //
    public boolean mouseDown (Event evt, int x, int y)
                       // mouse click position is (x,y)
    {
        int  which_mouse = 0;  //unset
        predAction = UNSET;  //make sure it only sets for a LINK tool

	// "real" position, from offset of DrawArea (related to
	// scrollbar value).  (0, 0) is with respect to the current
	// "view" over the underlying component, given by the scrolling.
	//
        x = x + offset_x;
        y = y + offset_y;

	// calculate which mouse button was pushed (or equivalent)
	//
	which_mouse = MouseHelper.which_mouse_button(evt);
            //System.out.println ("which mouse is DOWN: " + which_mouse);

        switch (tb.getCurrentTool())
        {
            case   Tools.CREATE:
		if (which_mouse == 1)
		{
                    // create new icon
                    active = createIcon(x,y);
		}
		else  if (which_mouse == 4)
                //button 1 with the shift or control key down
		//(if one of these is bound to a function the window manager
		//is using the application won't get it.  Hopefully, this
		//will allow the other to work.)
		{
		    // rename this icon
		    renameIconObject (x,y);
		}
		else  if (which_mouse == 2)
		{
		    // delete this icon
		    deleteIconObject (x,y);
		}
		else  if (which_mouse == 3)
		{
		    // currently a no-op
		}
		else  ; //noop

                break;

            case   Tools.MOVE:

                // any mouse button down
		activateObject (x, y);

                break;

            case   Tools.LINK:

		if (which_mouse == 1)
		{
		    beginCreateLink (x, y);
		}
		else  if (which_mouse == 4)  //button 1 with the shift key down
		{
		    // rename this icon
		    renameLinkObject (x, y);
		}
		else  if (which_mouse == 2)
		{
		    // delete this icon
		    deleteLinkObject (x, y);
		}
		else  if (which_mouse == 3)
		{
		    // reverse link
		    reverseLinkByMouse (x, y);
		}
		else  ; //noop

                break;

            default:
                return false;
        }
        return true;
    }

    public boolean mouseUp (Event evt, int x, int y)
    {
	// "real" position, from offset of DrawArea (related to
	// scrollbar value).
	//
        x = x + offset_x;
        y = y + offset_y;

        switch (tb.getCurrentTool())
        {
            case Tools.CREATE:
		break;

            case Tools.LINK:

		// calculate which mouse button was pushed (or equivalent)
		// (only care for LINK tool, unless we add more functions)
		//
		int which_mouse = MouseHelper.which_mouse_button(evt);
                        //System.out.println ("which mouse is UP " +
                        //     "(only note with Link tool): " + which_mouse);

		if (which_mouse == 1)
		{
		    createLink (x, y);
		}

                break;

            case Tools.MOVE:
                break;

            default:
    	        System.out.println ("ERROR!:  Expected another tool to exist");
        }        // end 'switch'

        return true;
    }
  
    public boolean mouseDrag (Event evt, int x, int y)
    {
        x = x + offset_x;
        y = y + offset_y;
        
        switch (tb.getCurrentTool())
        {
            case  Tools.CREATE:
                    break;

            // Draw the link line
            case  Tools.LINK:
		    //should be 1st mouse button only
		    int which_mouse = MouseHelper.which_mouse_button(evt);
                           //System.out.println ("which mouse is DRAGGED " +
                           //     "(only note with Link tool): " + which_mouse);

		    if (which_mouse == 1)
		        showPendingLink (x, y);
                    break;

            // Move the Icon around
            case  Tools.MOVE:
                    //any mouse button
		    doIconMove (x, y);
                    break;

            default:
    	         System.out.println ("ERROR!:  Another tool expected to exist");
        }
        return true;
    }

    public GraphObject hitTest (int x, int y)
    {
        GraphObject obj = null;
      
        // perform hit test on the icons and links.
        // Do in the reverse order that they were drawn
	// so you select top objects first

        // Icons first
        for (int i=(icons.size()-1); i >= 0; i--)
        {
            obj = (Icon) icons.elementAt(i);
            if (obj.mouseWithin(x,y))
                return obj;
        }

        // then Links
        for (int i=(links.size()-1); i >= 0; i--)
        {
            obj = (Link) links.elementAt(i);
            if (obj.mouseWithin(x,y))
                return obj;
        }

        return null;
    }

    public boolean changeText(String str)
    {
        // Do only for Icons ???
        if (active == null)
            return false;

        // Change Icon name
        active.renameGO(str);

	// TO DO:
        // change column title if active is a variable and resultArea exists
        //if ((active instanceof Icon) && (active.getIsVariable()))

        paint (getGraphics());
        return true;
    }

    void activateObject (int x, int y)
    {
	// find which object got hit
	GraphObject  temp = hitTest(x,y);

        // Make it the active object
        selectObject(temp);

        // Set offset if it is an icon
        if (temp instanceof Icon)
            ((Icon)temp).setOffset(x,y);

        requestFocus();
	paint (getGraphics());
    }

    // Does all the work of updating the window when a new GraphObject
    //    (or null) is selected.
    public void selectObject(GraphObject obj)
    {
        // unhighlight currently selected graphObject
        if (active != null)
            active.unselectGO();

        active = obj;

        if (obj == null)
        {
            // Nothing selected
	    tb.deactivateIconDisplays();
	    parent.setMenuSensitivity (false, false, false);
	    parent.pn.displayNumAnswersToQueryMsg();
        }
        // Link Selected
        else if (obj instanceof Link) 
        {
            // highlight Link
            active.selectGO();

            // deactivate text string and checkbox
	    tb.deactivateIconDisplays();
	    parent.setMenuSensitivity (false, true, false);
        }
        // Icon Selected
        else if (obj instanceof Icon)
        {
            // highlight icon
            active.selectGO();

            // move to first position in icon vector
            // so icon pops to the top if covered.
            icons.removeElement(active);
            icons.addElement(active);

            // move this Icon's Links to the top too.
            for (Enumeration e = ((Icon)active).links.elements();
                           e.hasMoreElements(); )
            {
                Link ln = (Link)e.nextElement();
                links.removeElement(ln);
                links.addElement(ln);
            }

	    Icon  activeIcon = (Icon)active;

            // set text field to string in new icon; set "variable" button
	    // display status
	    tb.activateIconDisplays (activeIcon.getObjName(),
                                     activeIcon.getIsVariable());

            // Change menus
	    parent.setMenuSensitivity (true, false, activeIcon.getIsVariable());
		//obj. menu becomes sensitive; link menu is made insensitive
        }

        // Redraw 
        paint (getGraphics());
    }

    //when we want to only allow deletion of an Icon, not
    //a link, we call this routine as a front ent to deleteObject()
    //
    void deleteIconObject (int x, int y)
    {
	GraphObject currOb = hitTest(x,y);
	if (currOb instanceof Icon)
	{
	    selectObject (currOb);
	    deleteObject();
	}
	else
	    selectObject(null);
    }

    //when we want to only allow deletion of a Link, not
    //an icon, we call this routine as a front ent to deleteObject()
    //
    void deleteLinkObject (int x, int y)
    {
	GraphObject currOb = hitTest(x,y);
	if (currOb instanceof Link)
	{
	    selectObject (currOb);
	    deleteObject();
	}
	else
	    selectObject(null);
    }

    //deletes object made currently active one
    //
    public boolean deleteObject()
    {
        if (active != null)
        {
            // Delete the Object
            active.deleteGO();
            selectObject(null);    //does redraw

            return true;
        }
        else
            return false;
    }

    void renameIconObject (int x, int y)
    {
	GraphObject currOb = hitTest(x,y);
	if (currOb instanceof Icon)
	{
	    selectObject (currOb);

	    // Prepare for an action on current Icon
	    tb.activateTextField();

	    renameObject();
	}
	else
	    selectObject(null);
    }

    void renameLinkObject (int x, int y)
    {
	GraphObject currOb = hitTest(x,y);
	if (currOb instanceof Link)
	{
	    selectObject (currOb);
	    renameObject();
	}
	else
	    selectObject(null);
    }

    void renameObject()
    {
	if (active == null)
	    return;
	else   if (active instanceof  Icon)
        {
             String newStr = tb.getTextContents();
	     active.renameGO (newStr);
        }
	else   if (active instanceof  Link)
	{
	    predAction = RENAMELINK;
            askForPredicate();
	}
	paint (getGraphics());
    }

    void reverseLinkByMouse (int x, int y)
    {
	GraphObject currOb = hitTest(x,y);
	if (currOb instanceof Link)
	{
	    selectObject (currOb);
	    reverseLink();
	}
	else
	    selectObject(null);
    }

    void reverseLink()
    {
        if (active instanceof Link)
        {
            ((Link)active).reverse();
            paint (getGraphics());
        }
    }

    // x and y are mouse coords.
    void doIconMove (int x, int y)
    {
	if ((active != null) && (active instanceof Icon))
	{
            active.X = x - ((Icon)active).get_dx();
            if (active.X < 0)
                active.X = 0;
            if (active.X >= MAX_SIZE)
                active.X = MAX_SIZE-1;
            active.Y = y - ((Icon)active).get_dy();
            if (active.Y < 0)
                active.Y = 0;
            if (active.Y >= MAX_SIZE)
                active.Y = MAX_SIZE-1;
            paint (getGraphics());
	}
    }

    // x and y are mouse coords.
    void showPendingLink (int x, int y)
    {
	// active is the from (or domain) Icon
        if ((active != null) && (active instanceof Icon))
	{
            Graphics grf = getGraphics();
            paint (grf);
            grf.setColor (Link.LINE_COLOR);
            grf.drawLine (x - offset_x, y - offset_y,
                      active.X - offset_x, active.Y - offset_y);
            active.drawGO (grf, offset_x, offset_y);
	}
    }

    void changeVariable (boolean val, boolean togglep)
    {
        if (active instanceof Icon) 
        {
            Icon icon = (Icon)active;
	    icon.changeVariable (val, togglep);

	    boolean newState = icon.getIsVariable();
            tb.setVariableState (newState);
            parent.isvariableMenuItem.setState (newState);

            paint (getGraphics());
        }
    }

    void updateFont(Font f, FontMetrics fm)
    {
        this.f = f;
        this.fm = fm;

        Enumeration e = links.elements();
        while (e.hasMoreElements()) 
            ((GraphObject)e.nextElement()).updateFont(f, fm);

        e = icons.elements();
        while (e.hasMoreElements())
            ((GraphObject)e.nextElement()).updateFont(f, fm);

        paint (getGraphics());
    }

    // Return the variable with the give name
    public Icon getVariable (String str)
    {
	// System.out.println ("       " + str);

        String   strWithoutPrefix;
        Character first = new Character (str.charAt(0));
        Character oneToMatch = new Character (Icon.VARIABLE_PREFIX);
	if (first.equals (oneToMatch))
	    strWithoutPrefix = str.substring(1);
	else
	    strWithoutPrefix = str;

	// System.out.println ("       " + strWithoutPrefix);

        Enumeration e = icons.elements();
        Icon icon;    
        while (e.hasMoreElements())
        {
            icon = (Icon)e.nextElement();
	    // System.out.println ("       this icon's name: <"
            //              + icon.getObjName() + ">");

            if ((icon.getObjName()).equals (strWithoutPrefix))
                return icon;
        }
        return null;
     }

    // Remove all bindings from the query graph
    void unbindVariables()
    {
        Enumeration e = icons.elements();
        while (e.hasMoreElements()) 
            ((Icon)e.nextElement()).changeBinding(null);
    }

    void paintDrawArea()
    {
        paint (getGraphics());
    }

    public void printDebug()
    {
        // Print Icons
        System.out.println("");
        System.out.println(icons.size() + " icons");
        for (int i=0; i<icons.size(); i++) 
        {
            Icon ic = (Icon)icons.elementAt(i);
            ic.debug(" ");
        }

        System.out.println(links.size() + " links");
        for (int i=0; i<links.size(); i++)
        {
            Link l = (Link)links.elementAt(i);
            l.debug(" ");
        }
        System.out.println("");
    }
}
