/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package  parka;

import   java.awt.*;

class FrameData extends Object
                implements  Runnable
{
    private String       frameName;
    private String       query;
    private ParkaNet     pn;
    private BrowserWin   bwin;
    private int          numAnswersTotal;
    private String       errMsg;
    private int          queryType;  //should have one of the values below
    private boolean      noerror;

    // values are to be disjoint with values in PredicateData for
    // analogous predicate query types
    // Both sets together are used to tell the server what to return
    static final int   PRED_UNRESTRICT     = 4;
    static final int   PRED_UNRESTRICT_IN  = 5;
    static final int   PRED_UNRESTRICT_OUT = 6;

    FrameData (ParkaNet pnet, BrowserWin browser, int type)
    {
	frameName = null;
	query = null;
	pn = pnet;
	bwin = browser;
	errMsg = "";
	queryType = type;
        if ((queryType >= PRED_UNRESTRICT) &&
                                (queryType <= PRED_UNRESTRICT_OUT))
            noerror = true;
	else
	{
	    noerror = false;
	    System.out.println ("Programmer Error:  " +
                    "Non-existent browser query type chosen");
	}
    }

    FrameData (String objectname, ParkaNet pnet, BrowserWin browser, int type)
    {
	frameName = objectname;
	query = null;
	pn = pnet;
	bwin = browser;
	errMsg = "";
	queryType = type;
        if ((queryType >= PRED_UNRESTRICT) &&
                                (queryType <= PRED_UNRESTRICT_OUT))
            noerror = true;
	else
	{
	    noerror = false;
	    System.out.println ("Programmer Error:  " +
                    "Non-existent browser query type chosen");
	}
    }

    // need to call this routine before calling 'sendQuery()' if first
    // constructor was used.
    //
    void setFrameName (String fname)
    {
	frameName = fname;
    }

    // 'query' is filled if frameName exists;
    // otherwise it is left null.
    //
    public void run()
    {
	if (frameName != null && noerror)
	{
	    int   status;

	    bwin.setCursor (Frame.WAIT_CURSOR);

	    query = String.valueOf(queryType) + " " +
                       String.valueOf(BrowserWin.FRAMECONST) + " "
                               + frameName + "\n";

	    status = pn.submitBrowsingQuery (query);
	    errMsg = pn.getBrowserMsg();
/* TEST
	    System.out.println ("   get err msg: [" + errMsg + "]");
*/
	    if (status >= -1)    // 0 is no error, all data;
                                 // -1 is no error, sample of data
	    {
	        // Note that a predicate must have a before and after node,
	        // unlike the case with nodes-- they may or may not have
	        // both a before and after predicate.   Nodes might have
	        // only one of those.
	        //
		String[][]  answerLists = pn.getBrowserAnswers();

		if (answerLists != null)
		{
		    for (int i = 0; i < answerLists.length; i++)
		    {
		        String[]  column = answerLists[i];
		        bwin.fillList (column);
	            }
	        }
		else
		{
                    // This seems to always be pgmmer error, when it occurs,
		    // but we will leave it in, in case.
		    //
		    errMsg = "Empty answer list";
		    bwin.displayForBrowser (errMsg);
// TEST
System.out.println ("This is probably a programmer error.  Check it out.");
		}
	    }
	    if (errMsg.length() > 0)
		bwin.displayForBrowser (errMsg);

	    bwin.setCursor (Frame.DEFAULT_CURSOR);
	}
    }
}
