/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
/**********************************************************************
  GraphObject
  **********************************************************************/
package parka;

import java.awt.*;
import java.util.*;

abstract class GraphObject extends java.lang.Object
{
    // Coordinates
    protected int X;
    protected int Y;

    // draw area pointer
    protected DrawArea draw;

    // width & height
    int width;
    int height;
    int descent;

    // Icon name or link predicate
    String str;

    // font
    Font f;
    FontMetrics fm;
    //String fontname;  // font name
    //int fs;  // font size

    // Constants
    protected final static int XBUF = 6;
    protected final static int YBUF = 2;

    // *******
    // Methods
    // *******

    // used by Icon constructor
    GraphObject (DrawArea  da, int xval, int yval)
    {
	super();
        str = "";
	draw = da;
        X = xval;
        Y = yval;
        width = 0;
        height = 0;
        descent = 0;
    }

    // used by Link constructor
    // we use this one rather than the one above, since
    // the Icons could (by programming error) be null.
    // See Link class.
    protected GraphObject (DrawArea  da, Icon from, Icon to)
    {
	super();
        str = "";
	draw = da;
        X = Link.getNewX (from, to);
        Y = Link.getNewY (from, to);
        width = 0;
        height = 0;
        descent = 0;
    }

    String getObjName()
    {
	return   str;
    }

    // Get text for the graphics
    abstract String getObjLabel();

    // Draw the GraphObject
    abstract void drawGO (Graphics g, int offset_x, int offset_y);

    // For selecting the GraphObject
    abstract void selectGO();

    // For deselecting the GraphObject
    abstract void unselectGO();

    // Delete the GraphObject
    abstract void deleteGO();

    // Determine if the coordinate is in the GraphObject
    public boolean mouseWithin (int x, int y)
    {
        return ((x > X-width/2-XBUF)  && (x < X+width/2+XBUF) &&
                (y > Y-height/2-YBUF) && (y < Y+height/2+YBUF));
    }

    // Update the font and dependent information for the GraphObject
    public void updateFont (Font f, FontMetrics fm)
    {
        /*
        fontname = win.fontname;
        fs = win.fontsize;
        f = new Font(fontname, Font.PLAIN, fs);
        fm = applet.getFontMetrics(f);
        */

        this.f = f;
        this.fm = fm;

        width = resetWidth();
	// System.out.println ("width changed (update font):  " + width);

        descent = fm.getDescent();
        height = fm.getAscent()+descent;
    }

    int resetWidth()
    {
	return  (fm.stringWidth (getObjLabel()));
    }

    // Rename the GraphObject (Change the string in the Object)
    void renameGO (String newName)
    {
	str = newName;
        // Update the Object's size
        width = resetWidth();

	// System.out.println ("width changed (rename):  " + width);
    }

    // Debugging function
    abstract void debug(String str);
}
