/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */

/**********************************************************************
  icon
  **********************************************************************/
package parka;

import java.awt.*;
import java.util.*;

public class Icon extends parka.GraphObject
{
    // offset
    private int dx;
    private int dy;
  
    // color
    final static Color DEF_COLOR = new Color(180,180,180);
    final static Color HI_COLOR = Color.yellow;

    // The background color for the Icon - changes when un/selected.
    private Color c;

    public final static char  VARIABLE_PREFIX = '?';

    // is this a variable or constant
    private boolean isvariable;

    // Set of Links associated with this object.
    Vector links;

    // The binding can be "".
    private String binding;
    private String iconLabel;
	//when binding is not null, it is combined with superclass's 'str'
        //(the object's name) and the binding, here.  Used for display only.

    // *******
    // Methods
    // *******

    Icon (DrawArea MyDraw, int x, int y, Font f, FontMetrics fm)
    {
        super (MyDraw, x, y);
        isvariable = true;  // variable by default
        binding = null;       // no binding initially
        iconLabel = null;
        str = "untitled";
        updateFont(f, fm);  // set the font and size information
        links = new Vector();
        dx = 0;
        dy = 0;
        c = DEF_COLOR;      //background color of icon
    }

    int get_dx()
    {
	return  dx;
    }

    int get_dy()
    {
	return  dy;
    }

    // Draw the GraphObject
    void drawGO (Graphics g, int offset_x, int offset_y)
    {
        // draw the base of the icon
        g.setColor(c);
	//positions:
        int  top = X - width/2 - XBUF - offset_x;
	int  left = Y - height/2 - YBUF - offset_y;
	int  localwidth = width + XBUF*2;
	int  localheight = height + YBUF*2;

        // if it's a variable, use a round-rect
        if (isvariable)
        {
            g.fillRoundRect (top, left, localwidth, localheight,
                                       height, height);
            g.setColor (Color.black);
            g.drawRoundRect (top, left, localwidth, localheight,
                                       height, height);
        }
        else
        {
            g.fillRect (top, left, localwidth, localheight);
            g.setColor (Color.black);
            g.drawRect (top, left, localwidth, localheight);
        }
        g.setFont(f);

	//System.out.println ("drawing label string now; width = " + width);

        g.drawString (getObjLabel(), X - width/2 - offset_x,
                       Y + height/2 - descent - offset_y);
    }

    // Save the offset distance between the mouse and the object coordinates
    void setOffset (int x, int y)
    {
        dx = x - X;
        dy = y - Y;
    }

    // Delete the GraphObject
    void deleteGO()
    {
        Vector tmp = (Vector)links.clone();
// XXX why is vector cloned??

        // Delete the Links associated with this Icon
        for (Enumeration e = tmp.elements();  e.hasMoreElements(); )
        {
            Link ln = (Link)e.nextElement();
            ln.deleteGO();
        }

        // Delete self from DrawArea list
        draw.icons.removeElement(this);
    }

    // Add Link
    void addLink(Link ln)
    {
        links.addElement(ln);
    }

    // Delete Link
    void deleteLink(Link ln)
    {
        links.removeElement(ln);
    }

    // For selecting the GraphObject
    void selectGO()
    {
        c = HI_COLOR;
    }

    // For deselecting the GraphObject
    void unselectGO()
    {
        c = DEF_COLOR;
    }

    // Variable/Constant status can toggle
    void  changeVariable (boolean val, boolean togglep)
    {
        if (togglep)
            isvariable = !isvariable;
        else
            isvariable = val;
    }

    boolean getIsVariable()
    {
	return  isvariable;
    }

    // overrides parent's routine
    String getObjLabel()
    {
	if (iconLabel == null)
	{
	    // System.out.println ("object label is null -- returns [" + str +
                      //          "]; width = " + width);
	    return  str;
	}
	else
	{
	    // System.out.println ("returns object label [" + iconLabel +
                      //          "]; width = " + width);
	    return  iconLabel;
	}
    }

    // returns variable name without bindings, if there are any
    // while bindings to the variable exist, the super class's 'str'
    // includes the var name and the binding.  In this class, we
    // remember the original var name in 'origVarName'.  When the
    // bindings are removed, 'origVarName' is made null, and 'str' is
    // restored.
    private void setBindingAppearance (String obname)
    {
        if (isvariable && (binding != null) && (iconLabel == null))
        {
            iconLabel = obname + ": " + binding;
        }
        else  if (isvariable && (binding == null) && (iconLabel != null))
        {
	    iconLabel = null;
        }
    }

    //overrides parent's routine
    void renameGO (String newstr)
    {
	setBindingAppearance (newstr);
	// Update the Icon size
	super.renameGO (newstr);
    }

    //set binding value
    void changeBinding (String newstr)
    {
        binding = newstr;
	setBindingAppearance (str);
	width = resetWidth();
    }

    public void debug (String str)
    {
        System.out.println("");
        System.out.println(str + "Icon: " + this.str);
        System.out.println(str + "Icon label: " + iconLabel);
        System.out.println(str + " " + links.size() + " links");
        for (int i=0; i<links.size(); i++) {
            Link ln = (Link)links.elementAt(i);
            ln.debug(str + "  ");
        }
    }
}
