/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
/**********************************************************************
  link
  **********************************************************************/
package parka;

import java.awt.*;
import java.util.*;

public class Link extends parka.GraphObject
{
    Icon receipt;   // Range Icon
    Icon owner;     // Domain Icon - where the link is stored

    Color c;

    // color
    final static Color DEF_COLOR = Color.white;
    final static Color HI_COLOR = Color.yellow;
    final static Color LINE_COLOR = Color.blue;
    final static Color RANGE_COLOR = Color.red;

    final static int ARROW_RADIUS = 3;

    // *******
    // Methods
    // *******

    Link(DrawArea MyDraw, Icon o, Icon r, String s, Font f, FontMetrics fm)
    {
        super (MyDraw, o, r);
	    //in case o or r is null, we use this constructor, which
	    //calls the static 'getNew...()' routines below to calculate
	    //X and Y in GraphObject.
        receipt = r;
        owner = o;
        str = s;
        c = DEF_COLOR;

        updateFont(f, fm);  // set the font and size information
        owner.addLink(this);
        receipt.addLink(this);
    }

    static int getNewX (Icon from, Icon to)
    {
	if ((from != null) && (to != null))
	    return  ((from.X + to.X) / 2);
	else
	{
	    System.out.println ("Programming error:  " +
                     "One of nodes for link is missing");
	    return  0;
	}
    }

    static int getNewY (Icon from, Icon to)
    {
	if ((from != null) && (to != null))
	    return  ((from.Y + to.Y) / 2);
	else
	{
	    System.out.println ("Programming error:  " +
                     "One of nodes for link is missing");
	    return  0;
	}
    }

    void drawGO(Graphics g, int offset_x, int offset_y)
    {
        int Xloc, Yloc;
        float x1 = owner.X - offset_x;
        float y1 = owner.Y - offset_y;
        float x2 = receipt.X - offset_x;
        float y2 = receipt.Y - offset_y;
      
        X = (owner.X + receipt.X) / 2;
        Y = (owner.Y + receipt.Y) / 2;

        g.setFont(f);

        // draw line
        g.setColor(LINE_COLOR);
        g.drawLine((int)x1, (int)y1, X-offset_x, Y-offset_y);
        g.setColor(RANGE_COLOR);
        g.drawLine(X-offset_x, Y-offset_y, (int)x2, (int)y2);
        
        // Calculate location of arrow by finding the intersection
        // of the line and box.
        float xx, yy;
        xx = 0;
        yy = 0;

        // Check intersection with the top or bottom
        if (y1 != y2) {
            yy = y2 - (receipt.height / 2) - YBUF;
            if (y2 < y1)  
                yy += receipt.height + (2*YBUF);
            if (x1 == x2)  
                xx = x2;
            else
                xx = ((x2-x1)/(y2-y1)) * (yy-(((x2*y1) - (x1*y2))/(x2-x1)));
        }
    
        // If not, check for left or right side
        if ((xx < (x2 - (receipt.width / 2) - XBUF)) ||
             (xx > (x2 + (receipt.width / 2) + XBUF))) 
        {
            xx = x2 - (receipt.width / 2) - XBUF;
            if (x2 < x1) 
                xx += receipt.width + (2*XBUF);
            if (y2 == y1)
                yy = y2;
            else
                yy = (((y2 - y1) / (x2 - x1)) * xx) +
                                       (((x2 * y1) - (x1 * y2)) / (x2 - x1));
        }
    
        // Draw the arrow
        g.fillOval((int)xx-ARROW_RADIUS , (int)yy-ARROW_RADIUS,
                    ARROW_RADIUS*2, ARROW_RADIUS*2);

        // draw string
        Xloc = X - width/2 - offset_x;
        Yloc = Y - height/2 - offset_y;
        g.setColor(c);  // set background color
        //g.fillRect(Xloc, Yloc + 1, width, height - 1);
        g.fillRect(Xloc, Yloc, width, height);
        g.setColor(Color.black);   // use black ink
        //g.drawString(str, Xloc + 2, Yloc + width - 3);
        g.drawString(str, Xloc+1, Yloc + height - descent);
        //g.drawRect(Xloc, Yloc, width, height);
    }

    String getObjLabel()
    {
	return   str;
    }

    // Delete the GraphObject
    void deleteGO()
    {
        owner.deleteLink(this);
        receipt.deleteLink(this);
        draw.links.removeElement(this);
    }

    // 'newName' was chosen through the PredDialog class
    // void renameGO (String newName); -- uses parent's routine.

    // For selecting the GraphObject
    public void selectGO()
    {
        c = HI_COLOR;
    }

    // For deselecting the GraphObject
    public void unselectGO()
    {
        c = DEF_COLOR;
    }

    // Reverse the link
    public void reverse()
    {
        Icon tmp = receipt;
        receipt = owner;
        owner = tmp;
    }

    public void debug(String str)
    {
        System.out.println("");
        System.out.println(str + "Link: " + this.str);
        System.out.println(str + " " + owner.str);
        System.out.println(str + " " + receipt.str);
    }
}
