/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package  parka;

import  java.io.*;
import  java.awt.*;

class  NetKBInit  extends Object
                  implements  Runnable
{
    Thread    KBloader;
    ParkaNet  pn;

        // KB information
    private String    KBName;
    private boolean   useMem;
    private InputStream   instream;   //expected to be set up elsewhere
    private OutputStream  outstream;  //expected to be set up elsewhere

    private int  oldCursor;

    NetKBInit (ParkaNet  pnet, InputStream is, OutputStream os)
    {
	pn = pnet;
	KBloader = null;
	KBName = "";
 	instream = is;
	outstream = os;
	oldCursor = 0;    // unset
    }

    boolean  isThreadAlive()
    {
	//we don't really care if it's started or not; just if it exists
	//
	if (KBloader != null)
	    return  true;
	else
	    return  false;
    }

    void threadJoin()
    {
	if (KBloader != null)
	{
	    try   {
	        KBloader.join();
            }
            catch  (InterruptedException  e)  {
                System.out.println ("KB loader thread interrupted");
            }
        }
    }

    // Start Load a KB
    // With a GUI interface, this is the routine to call
    // in this class, after the constructor.  The work done
    // by this class starts from here.
    //
    void loadKB()
    {
	if (pn.isConnection())
        {
            KBloader = new Thread (this, "KBloaderThread");

            // Get list of KBs
            String[] kbs = getKBs();

            // Have user choose KB and memory/disk option
            KBDialog cbd = new KBDialog (pn.getWin(), this, kbs);
	    if (ApplType.getIsVersion1_0())
		((Window)cbd).show();
	    else
                cbd.show();
	}
	else
            (pn.getWin()).displayMessage (ParkaNet.NOCONNMSG);
    }

    // KB loading occurs in a separate thread, until done.
    public void run()
    {
	// System.out.println ("Entered 'run() (for thread to load KB)'");

        // load KB
        pn.openKB (KBName, useMem);   //this part may take a good while

	// this part should be quick with any KB:
        if (pn.isKBOpen())
        {
            // Restore old cursor
            (pn.getWin()).setCursor (oldCursor);

            (pn.getWin()).displayMessage("Finished Loading " + KBName);
	}
	// else message displayed from ParkaNet.openKB() is just fine.

	KBloader = null;
    }

    void loadKBbyName (String kb, boolean useMem)
    {
	// System.out.println ("Entered 'loadKBbyName()'");

        // Set up wait cursor
        oldCursor = (pn.getWin()).getCursorType();
        (pn.getWin()).setCursor (Frame.WAIT_CURSOR);

	KBName = kb;
	this.useMem = useMem;
	/* System.out.println ("In 'loadKBbyName()': KBname is: [" + KBName +
                       "] ; useMem = " + useMem);
        */
	/* Carloyn had removed a line that cause a NullException error... */
	if (KBloader == null)
	  KBloader = new Thread (this, "KBloaderThread");
	KBloader.start();
    }

    private String[] getKBs()
    {
        (pn.getWin()).displayMessage("Retrieving KB list");
	String[]  kblist;

        // Returns a list of available Knowledge Bases
	if (pn.isConnection())
        {
            Command  c = new Command (pn, Command.GETKBNAMES, "");
	    c.sendCommand();
            Response r = new Response (pn);
	    r.computeResponse();
	    if (r.getStatus() > 0)
	    {
	        r.makeTokens();
	        kblist = r.getTokenList();
	    }
	    else
	    {
	        kblist = new String[0];
		(pn.getWin()).displayMessage (r.getResponse());
	    }
        }
	else
        {
	    (pn.getWin()).displayMessage (ParkaNet.NOCONNMSG);
	    kblist = new String[0];
        }

        return   kblist;
    }
}
