/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package parka;

import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.Frame;  //for Cursor
import java.awt.Window; //for dialogs

/**********************************************************************
  ParkaNet - networking interface for Parka
  **********************************************************************/
public class  ParkaNet  extends java.lang.Object
{
    // KB information
    private String KBName;
    private boolean usingMem;
             // 'useMem' refers to using memory vs. disk in internal
             // Parka operations, ie, joins.
    private String[] predicates;

    private ParkaWin parent;

    private Socket       socket;
    private InputStream  instream;
    private OutputStream outstream;

    private NetKBInit  kbLoader;
    private boolean    kbOpen;
    private boolean    newPredList;

    // Hostname
    private String hostname;

    // Port number
    private int port;

    // Query results
    private int    numTotalAnswersToQuery;
           //number of total tokens from the server is going to be--
           //(if all of the answers were requested at once, rather than
           // in batches):
           //numTotalAnswersToQuery * getNumVariables() --
           //                  ie, num Rows * num Cols
    private int    varStatus;
    private String     numAnswersMsg;
    private String[]   answers;
    private String[]   vars;

    private static int   HAS_VARS = 1;
    private static int   NO_VARS  = 2;
    private static int   VARS_OTHER_STATUS  = 3;

    // Browser query results
    private String[][]  browserAnswers;
    private String      browserMsg;

    static final String  NOCONNMSG = "Not connected to server";

    /*******
     Methods
     *******/

    public ParkaNet (ParkaWin parent)
    {
        // constructor
        this.parent = parent;
        socket = null;
	instream = null;
        outstream = null;

	initKBVars();
	initBrowserData();

	//default values:
	hostname = "theophilos.cs.umd.edu";
	port = 7990;
    }

    //some of these are also reinitialized with a new query
    private void initKBVars()
    {
        setKBStatus (false);
        KBName = null;
	usingMem = true;   //may be reset later by chosen KB. Default.
        numTotalAnswersToQuery = 0;
	numAnswersMsg = null;
        predicates = null;
        answers = null;
        vars = null;
	varStatus = VARS_OTHER_STATUS;
	newPredList = true;   // 'predicates' not init'd yet
    }

    void  initBrowserData()
    {
        browserAnswers = null;
	browserMsg = "";
    }

    void clearNumAnswersToQueryMsg()
    {
	numAnswersMsg = null;
    }

    void displayNumAnswersToQueryMsg()
    {
	if (numAnswersMsg != null)
	    parent.displayMessageKS (numAnswersMsg);
    }

    ParkaWin  getWin()
    {
	return   parent;
    }

    InputStream  getInStream()
    {
        return  instream;
    }

    OutputStream  getOutStream()
    {
        return  outstream;
    }

    public void connect()
    {
        // connect to Parka
        parent.displayMessage("Connecting to Parka");
        try 
        {
            // close socket if one exists
            if (this.isConnection())
                this.disconnect();

            // Create new socket and streams
            socket = new Socket (hostname, port);
            instream = socket.getInputStream();
            outstream = socket.getOutputStream();
            parent.displayMessage("Connection -- Waiting");
		// listening socket -- successful connect

	    is_connection_accepted();
        }
        catch (UnknownHostException uhe) 
        {
            parent.displayMessage("Connect Failed: Unknown Host Error");
        }
	/* The following code is Java 1.1 */
	/*
        catch (BindException be) 
        {
            parent.displayMessage("Connect Failed: " + be.getMessage());
	//TEST
	    System.out.println ("BindException");
        }
        catch (ConnectException ce) 
        {
            parent.displayMessage("Connect Failed: " + ce.getMessage());
	//TEST
            System.out.println ("Check if server is running " +
                           "or port is correct");
        }
	*/
        catch (SocketException se) 
        {
            parent.displayMessage("Connect Failed: Socket Failure");
	//TEST
	    System.out.println (se.getMessage());
        }
        catch (IOException ioe)
        {
            parent.displayMessage("Connect Failed: IO Error");
	//TEST
	    System.out.println (ioe.getMessage());
        }
	catch (IllegalArgumentException  iae) //possible with illegal port#
	{
            parent.displayMessage ("Connect Failed: " + iae.getMessage());
	//TEST
	    System.out.println (iae.getMessage());
	}

/* TEST
	System.out.println ("Leaving 'connect()'");
*/
    }

    private void is_connection_accepted()
    {
/* TEST
	System.out.println ("In is_connection_accepted()");
*/
        Response  r = new Response (this);
	r.computeResponse();
/* TEST
	System.out.println ("Response:  " + r.getResponse());
*/
	parent.displayMessage (r.getResponse());
    }

    public void disconnect()
    {
        // disconnect from Parka if connected
        if (isConnection())
        {
            // close open KB, if any
            if (isKBOpen())
	    {
		if (kbLoader.isThreadAlive())
		    kbLoader.threadJoin();
                closeKB();
	    }

            // now disconnect
            parent.displayMessage("Disconnecting from Parka");
            try
            {
                // Tell parka server we are exiting;
		// parka doesn't give a response
		Command c = new Command (this, Command.DISCONNECT, "");
                c.sendCommand();

                // Close streams
                instream.close();
                outstream.close();
                socket.close();
            
                instream = null;
                outstream = null;
                socket = null;
            }
            catch (IOException e1)
            {
                // Close Error
                System.out.println("IO Error in disconnect()");
            }
        }
    }

    // Returns true if there is a connection to PARKA open
    public boolean isConnection()
    {
        return ((socket != null) && (instream != null)
                            && (outstream != null));
    }

    // Change hostname and port used to connect to PARKA
    public void changeParkaHost(String newhost, int newport)
    {
        hostname = newhost;
        port = newport;
        
        // change PARKA connection
        this.disconnect();
        this.connect();
    }

    // Create PortDialog so user can change hostname and portnumber
    public void doPortDialog()
    {
        PortDialog pdialog = new PortDialog (parent, this, hostname, port);
	if (ApplType.getIsVersion1_0())
            ((Window)pdialog).show(); //Netscape thinks we want Dialog.show()
             //which doesn't exist until java 1.1
	else
            pdialog.show();
    }

    public synchronized boolean isKBOpen()
    {
	return   kbOpen;
    }

    public synchronized void setKBStatus (boolean status)
    {
	kbOpen = status;
    }

    // initialize connection to a new KB
    //
    public void initKBLoading()
    {
      /* added by Carolyn for multiple Parka server? */
        if ((kbLoader != null) && kbLoader.isThreadAlive())
	  kbLoader.threadJoin();
	if (isKBOpen())
	  closeKB();

	if (isConnection())  //assures the 2 streams already exist
	{
	    kbLoader = new NetKBInit (this, instream, outstream);
		//this class requires in/out streams to already be set up.
	    kbLoader.loadKB();
	}
	else
            parent.displayMessage (NOCONNMSG);
    }

  /* initialize connection to a new KB without opening a dialog to 
     request info from the user (Heflin, 5/21/98) */
    public void initLoadKBbyName(String newkbName, boolean memoryp)
    {
      /* added by Carolyn for multiple Parka server? */
        if ((kbLoader != null) && kbLoader.isThreadAlive())
	  kbLoader.threadJoin();
	if (isKBOpen())
	  closeKB();

	if (isConnection())  //assures the 2 streams already exist
	{
	    kbLoader = new NetKBInit (this, instream, outstream);
		//this class requires in/out streams to already be set up.
	    kbLoader.loadKBbyName (newkbName, memoryp);
	}
	else
            parent.displayMessage (NOCONNMSG);
    }

    // this routine is used from within a separate thread; so
    // we need to be careful about synchronization and interference.
    //
    public void openKB (String newkbName, boolean memoryp)
    {
	// System.out.println ("Entered 'openKB()'");

	if (isConnection())
        {
	    // System.out.println ("presence of connection is established.");

	    // not set from any other place:
	    KBName = newkbName;
	    usingMem = memoryp;
	    /* System.out.println ("   KBname = [" + KBName +
                        "] ; usingMem = " + usingMem);
            */
            // load a specific database
            parent.displayMessage("Loading " + KBName);

            Command  c = new Command (this, Command.OPENKB, KBName);
	    c.sendCommand();
            Response r = new Response (this);
	    r.computeResponse();
	    // System.out.println ("   returned status is: " + r.getStatus());

            if (r.getStatus() == 0)
            { 
                setKBStatus (true);
            }

            parent.displayMessage (r.getResponse());
        }
    }

    public void closeKB()
    {
        if (isKBOpen())
        {
	    if (kbLoader.isThreadAlive())
	        kbLoader.threadJoin();

	    if (isConnection())
            {
                // Close the current KB
                parent.displayMessage("Closing KB");

                Command  c = new Command (this, Command.CLOSEKB, "");
                c.sendCommand();
                Response r = new Response (this);
	        r.computeResponse();

		if (r.getStatus() >= 0)  // succeeds
	        {
		    //reinit when closed
    		    initKBVars();
		    initBrowserData();
		    parent.reinitAppWin();
		}

                parent.displayMessage (r.getResponse());
            }
	    else
                parent.displayMessage (NOCONNMSG);
        }
	else
            parent.displayMessage ("No KB is open");
    } 

    // 'links' are the predicates in the query graph
    public void submitQuery (Vector links)
    {
	if (!isConnection())
        {
	    parent.displayMessage(NOCONNMSG);
	    return;
	}
	if (!isKBOpen())
	{
	    parent.displayMessage("No open KB");
	    return;
	}

        kbLoader.threadJoin();
        parent.displayMessage("Submitting Query");

        // submit a query to parka engine
        StringBuffer querystring = new StringBuffer();
      
        /*  gather query info */
        Enumeration preds = links.elements();
        while (preds.hasMoreElements())
        {
            Link currlink = (Link)(preds.nextElement());
            querystring.append("(" + currlink.getObjName() + "#");

            if (currlink.owner.getIsVariable())
                querystring.append (String.valueOf (Icon.VARIABLE_PREFIX));
            querystring.append(currlink.owner.getObjName() + "#");
          
            if (currlink.receipt.getIsVariable())
                querystring.append (String.valueOf (Icon.VARIABLE_PREFIX));
            querystring.append(currlink.receipt.getObjName() + ")");
        }

        // error checking
	String   finalQuery = new String (querystring);
	if (finalQuery.equals(""))
	{
            parent.displayMessage ("No query formed");
	    return;
	}

        Command  c = new Command (this, Command.QUERY, finalQuery);
        c.sendCommand();
        Response r = new Response (this);
        r.computeResponse();

	numTotalAnswersToQuery = r.getStatus();

        if (numTotalAnswersToQuery < 0)   //then really have an error
        {
/*  TEST --
    what prints if we let 'r.getResponse()' print??

	    if (numTotalAnswersToQuery != Response.XCEPTION)
	    {
		// either boolean answer of "False", or no way to match
		// the query graph to any data in the database.  Could
		// include use of constants not in the database, or in
		// the database in a different context (eg. a predicate
		// used in a domain or range position)
	        //
                parent.displayMessage ("No results found");
	    }
	    else
*/
	        parent.displayMessage (r.getResponse());

            answers = null;
            vars = null;
        }
        else //if (numTotalAnswersToQuery >= 0) 
        {
            vars = getVariables();

	    if (varStatus == NO_VARS)
	    {
		numAnswersMsg = " Answer is TRUE";
                parent.displayMessage ("Done. " + numAnswersMsg);
	    }
	    else   //varStatus == HAS_VARS
	    {
		numAnswersMsg = " " + numTotalAnswersToQuery
                                       + " answers.";
                parent.displayMessage ("Done. " + numAnswersMsg);
	    }
        }
    }

    String getBrowserMsg()
    {
	return  browserMsg;
    }

    void printBrowserSampleMsgs (Vector coltype, Vector numans,
                                 Vector numshown)
    {
	int  vectsize = coltype.size();
	if (vectsize != numans.size())
	{
	    System.out.println ("Programmer Error: column type vector "
                 + "and number of answers in each column vector are "
                 + "not the same size.");
	    return;
	}

	StringBuffer msgBuf = new StringBuffer (
                     "Too much data; only a sample is used.");

	for (int i = 0; i < vectsize; i++)
	{
	    if ((i % 2) == 1)
		msgBuf.append ("\nColumn ");
	    else
	        msgBuf.append ("  Column ");
	    String  col = BrowserWin.getColumnName (
                                               (String)coltype.elementAt(i));
	    String  num = (String)numans.elementAt(i);
	    String  shown = (String)numshown.elementAt(i);
	    msgBuf.append (col);
	    msgBuf.append (" has ");
	    msgBuf.append (num);
	    msgBuf.append (" total answers. ");
	    msgBuf.append (shown);
	    msgBuf.append (" shown.");
	}
	browserMsg = new String (msgBuf);
    }

    // Browsing query has a specialized expected structure --
    // See comments on server side.
    // Returns contents of an error message, if any.
    //
    int  submitBrowsingQuery (String  query)
    {
	int  status;
	browserMsg = "";

        Command  c = new Command (this, Command.JBROWSERQUERY, query);
        c.sendCommand();
        BrowserResponse r = new BrowserResponse (this);
        r.computeResponse();
	status = r.getStatus();

	if (status >= -1)   // 0 is the highest possible
	{
	    r.makeTokens();
	    browserAnswers = r.getBrowserTokenList();
	    if (status == -1)   // sample of all the data only
	    {
	        Vector  columnType = r.getColType();
	        Vector  numberAns = r.getNumAnswers();
		Vector  numberShown = r.getNumAnswersShown();
	        printBrowserSampleMsgs (columnType, numberAns, numberShown);
	    }
	}
	if (status == -2)  //corrs to JB_INCOMPLETEDATA; get more data:
	{
	    r.makeTokens();

	    while (status == -2)  // 0 is a successful end
	    {
		c = new Command (this, Command.JBRWQUERYCONT, "");
		c.sendCommand();
		r.computeResponse();
		status = r.getStatus();

		if (status >= -2)
		{
	            r.makeMoreTokens();
		}
	    }
	    if (status >= -1)
	    {
	        browserAnswers = r.getBrowserTokenList();
	        if (status == -1)
		{
	            Vector  columnType = r.getColType();
	            Vector  numberAns = r.getNumAnswers();
		    Vector  numberShown = r.getNumAnswersShown();
	            printBrowserSampleMsgs (columnType, numberAns, numberShown);
		}
	    }
	    else
	        browserMsg = r.getResponse();
	}
	else  if (status == -3)
	    browserMsg = "No more data received; server out of memory";
	else  if (status == -4)
	    browserMsg = r.getResponse();

	return  status;
    }

    // Set up query results in the 'answers' array.
    // Should not call this routine if no query has previously
    // been sent.
    // get start..end answers from the server.
    // start must be > 0.
    //
    void getAnswersFromServer (int start, int end)
    {
	if (isConnection())
        {
	    //Nothing to process here:
	    if (numTotalAnswersToQuery <= 0)
		return;

	    if ((start < 1) || (end < 1))
            {
		System.out.println ("ParkaNet.getAnswersFromServer(): "
                   + "'start' 'and' end parameters must be greater than zero");
		return;
            }
	    if (end < start)
            {
		System.out.println ("ParkaNet.getAnswersFromServer(): "
		      + "'end' parameter cannot be less than 'start'");
		return;
            }

            Command  c = new Command (this, Command.GETBINDINGS,
                                   start + " " + end);
            c.sendCommand();
            Response r = new Response (this);
            r.computeResponse();
	    r.makeTokens();
	    answers = r.getTokenList();

	    /* TEST
	    if (answers == null)
	        System.out.println ("'answers' is null");
	    else
	        System.out.println ("'answers' has " + answers.length
                                             + " answers.");
	    */
	    // this will obtain tokens for row x column responses
            // num variables is number of columns; number of rows
            // is total/columns.
        }
        else
	{
            parent.displayMessage (NOCONNMSG);
	}
    }

    public String[] getAnswers()
    {
        return   answers;
    }

    public String[][] getBrowserAnswers()
    {
        return   browserAnswers;
    }

    public String[] getVariables()
    {
	if (isConnection())
        {
            Command  c = new Command (this, Command.GETVARIABLES, "");
	    c.sendCommand();
            Response r = new Response (this);
	    r.computeResponse();

	    if (r.getStatus() > 0)
	    {
	        r.makeTokens();
	        vars = r.getTokenList();
	        varStatus = HAS_VARS;
	    }
	    else  if ( (getNumTotalAnswersToQuery() == 0) &&
                       ((r.getResponse()).equals(QueryResults.NOVARSINQUERY)) )
                                 // [r.getStatus() == -1, in this case]
	    {
	        varStatus = NO_VARS;
		vars = null;
	    }
	    else
	    {
	        varStatus = VARS_OTHER_STATUS;
		vars = null;
	    }
        }
	else
        {
            parent.displayMessage (NOCONNMSG);
            vars = null;
        }

        return vars;
    }

// Not currently used:
    public void getStats()
    {
        // Get the size stats for the currently loaded KB
	if (isConnection())
        {
            Command  c = new Command (this, Command.PARKACONFIG, "");
	    c.sendCommand();
            Response r = new Response (this);
	    r.computeResponse();
	    // no display done
        }
	else
	    parent.displayMessage (NOCONNMSG);
    }

// Not currently used:
    public void getTime()
    {
        // Get the time information for the last query
	if (isConnection())
        {
            Command  c = new Command (this, Command.GETTIMINGS, "");
	    c.sendCommand();
            Response r = new Response (this);
	    r.computeResponse();
	    // no display done
        }
	else
	    parent.displayMessage (NOCONNMSG);
    }

    // Gets a list of predicates relevant to a particular database.
    // User class is expected to call this routine after the thread
    // loading the requested KB has successfully completed.
    //
    public String[] getPreds()
    {
	String[]  predList = null;

        // Get predicate list from PARKA
	if (isConnection())
        {
	    if (isKBOpen())
	    {
	        if (!kbLoader.isThreadAlive())
	        {
		// using the "KS" version of displayMessage so that the
		// active Icon is not disturbed (When the link/predicate
		// is created, we need to know this to assign the "from"
		// node. (KS == keep state).
                parent.displayMessageKS("Retrieving Predicates");

		boolean  morePredsToGet = true;
                boolean  inLoopAgain = false;
                while (morePredsToGet)
		{
		    String[] tmp, tmp2;

                    Command  c = new Command (this, Command.GETPREDICATES, "");
	            c.sendCommand();
                    Response r = new Response (this);
	            r.computeResponse();

                    // status == 0 if have no more preds to get.
		    if (r.getStatus() >= 0)
		    {
			// System.out.println ("get preds:  status > or = 0");
	                r.makeTokens();
	                tmp = r.getTokenList();
		    }
		    else //r.getStatus() < 0: error occurred while getting preds
		    {
			// System.out.println ("get preds:  status < 0");
		        predList = null;
		        parent.displayMessage (r.getResponse());
			break;
		    }

		    morePredsToGet = (r.getStatus() > 0);
		    if (inLoopAgain)
		    {
			// System.out.println ("  get preds:  " +
                        //              "Getting preds mult times thru loop");
			tmp2 = predList;
			predList = new String[tmp.length + tmp2.length];
                        // keep in original order:
			System.arraycopy (tmp2, 0,
			          predList, 0, tmp2.length);
			System.arraycopy (tmp, 0,
			          predList, tmp2.length, tmp.length);
		    }
		    else
			predList = tmp;

		    inLoopAgain = true;
		}
                // if r.getStatus() == 0, we have all the preds, and
                // everything is ok.
	        }
	        else
		    parent.displayMessageKS("Predicates not ready yet");
	    }
	    else
	    {
		parent.displayMessage("No KB open");
	        predList = null;  //assure null value when no connection
	    }
        }
	else
	{
	    parent.displayMessage (ParkaNet.NOCONNMSG);
	    predList = null;  //assure null value when no connection
	}
        /* TEST
        if (predList == null)
            System.out.println ("Predicates are null");
        else
            System.out.println (predList.length + " Predicates");
        */
        return    predList;
    }

    /* When we know that valid commands have been responded to, without
       error for the commands GETPREDICATES and GETVARIABLES, then it
       is valid to call the next two routines.    */

    // Returns the predicate array
    // used 'getPreds()' when KB was opened initially, to retrieve
    // the predicates from Parka
    //
    public String[] getPredicateList()
    {
	if (isKBOpen() && newPredList)
        {
	    if (kbLoader.isThreadAlive())
	        kbLoader.threadJoin();

	    predicates = getPreds();
	    newPredList = false;  //if predicates is still null,
                   // something is wrong.
        }
	else  if (!isKBOpen())
	    parent.displayMessage ("No KB is open");

        return predicates;
    }

    public String[] getVariableList()
    {
        return vars;
    }

    //same as number of columns for output display of query results
    //
    public int getNumVariables()
    {
	if (vars == null)
	    return   0;
	else
	    return   vars.length;
    }

    //total number of answers generated at the server
    //
    public int getNumTotalAnswersToQuery()
    {
	return  numTotalAnswersToQuery;
    }
}
