/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package parka;

import java.awt.*;
import java.util.*;

public class PortDialog extends ParkaDialog
{
    ParkaNet pnet;
    
    // Button constants
    public static final int CANCEL = 0;
    public static final int OK = 1;

    protected static final String TITLE = "Connection Request";
    protected static final String DIRECTION = "Enter hostname and port number of PARKA server";
    protected Label     directionLabel;

    protected TextField   hostField, portField;
    protected Label       hostLabel, portLabel;

    protected static final  String    hostStr = "Hostname: ";
    protected static final  String    portStr = "Port:     ";
    private   static final  int    HOSTSIZE = 40;
    private   static final  int    PORTSIZE = 10;

    protected   String   host;
    protected   int      port;

    /*******
     Methods
     *******/
    
    public PortDialog (ParkaWin parent, ParkaNet pn, String h, int p)
    {
        super (parent, TITLE);
        
	pnet = pn;
	host = h;
	port = p;

        // Specify a LayoutManager for it
        setLayout (new BorderLayout (5, 5));

	directionLabel = new Label (DIRECTION, Label.CENTER);
	add (BorderLayout.NORTH, directionLabel);
	     //tho we run under the 1.02 model when an applet, the
	     //compiler is 1.1.5; so const string identifier is recognized

	Panel upperP = makeUpperPanel();
	add (BorderLayout.CENTER, upperP);

	Panel lowerP = makeLowerPanel (1, 2);
	add (BorderLayout.SOUTH, lowerP);
              
        // Set the window to its preferred size.
        this.pack();
    }

    private Panel  makeUpperPanel()
    {
        Panel  newP = new Panel();

        // Specify a LayoutManager for it
	GridBagLayout  gridBag = new GridBagLayout();
	newP.setLayout (gridBag);
	GridBagConstraints gbc = new GridBagConstraints();
      
        // Setup text fields and labels
        hostLabel = new Label (hostStr, Label.RIGHT);
        portLabel = new Label (portStr, Label.RIGHT);
        hostField = new TextField (host, HOSTSIZE);
        portField = new TextField (String.valueOf(port), PORTSIZE);

        // If applet disable hostField editing???

	setGridBagConstraints (gbc, 0, 0, 1, 1, 40, 50,
                        GridBagConstraints.HORIZONTAL, 5, 5);
	gridBag.setConstraints (hostLabel, gbc);
	newP.add (hostLabel);

	setGridBagConstraints (gbc, 1, 0, 1, 1, 60, 0,
                        GridBagConstraints.HORIZONTAL, 5, 5);
	gridBag.setConstraints (hostField, gbc);
	newP.add (hostField);

	setGridBagConstraints (gbc, 0, 1, 1, 1, 0, 50,
                        GridBagConstraints.HORIZONTAL, 5, 5);
	gridBag.setConstraints (portLabel, gbc);
	newP.add (portLabel);

	setGridBagConstraints (gbc, 1, 1, 1, 1, 0, 0,
                        GridBagConstraints.HORIZONTAL, 5, 5);
	gridBag.setConstraints (portField, gbc);
	newP.add (portField);

	return   newP;
    }

    // Create a panel of buttons; center the row of buttons in the panel
    //
    private Panel  makeLowerPanel (int rows, int cols)
    {
        Panel  newP = new Panel();

        // Specify a LayoutManager for it
        newP.setLayout(new FlowLayout(FlowLayout.CENTER, 15, 15));

        newP.add (ok = new Button (OKlabel));
        newP.add (cancel = new Button (CANCELlabel));

	return  newP;
    }

    void setGridBagConstraints (GridBagConstraints constraints,
              int gx, int gy, int gwid, int ght, int wtx, int wty,
              int fill, int xinset, int yinset)
    {
        constraints.gridx = gx;
        constraints.gridy = gy;
        constraints.gridwidth = gwid;
        constraints.gridheight = ght;
        constraints.weightx = wtx;
        constraints.weighty = wty;
        constraints.fill = fill;
        constraints.insets = new Insets (yinset, xinset, yinset, xinset);
    }

    public synchronized Dimension preferredSize()
    {
	return  new Dimension (MIN_X + 50, MIN_Y + 10);
    }

    // Do Ok action
    protected void ok()
    {
        // Need to handle parse error here
        int portnum = Integer.parseInt(this.portField.getText());
        pnet.changeParkaHost(this.hostField.getText(), portnum);
    }
}
