/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package  parka;

import   java.awt.*;

class PredicateData extends Object
                    implements  Runnable
{
    private String       predName;
    private String       domName;
    private String       ranName;
    private String       query;
    private ParkaNet     pn;
    private BrowserWin   bwin;
    private int          numAnswersTotal;
    private String       errMsg;

    // values are to be disjoint with values in FrameData for
    // analogous predicate query types
    // Both sets together are used to tell the server what to return

    static final int   FRAME_UNRESTRICT    = 1;
    //have the range already; get domain:
    static final int   FRAME_RESTRICT_IN   = 2;
    //have the domain already; get range:
    static final int   FRAME_RESTRICT_OUT  = 3;
    //have both domain and range:
    static final int   DATAVERIFY          = 7;

    /* the built in predicates are hard-coded into the parka library */
    /* these must match those. */
/*      Not needed unless different handling from that of user defined
        predicates is reinstated.  
    private static final String[]  builtinStructPreds = { "isa", "instanceOf",
                      "subCat", "instance" }; 
*/
    private static final String[]  builtinQueryOnlyPreds = { "everyInstanceOf",
                      "everyInstance", "stringMatch", "gt", "lt", "eq",
                      "inRangeOf" };

    PredicateData (ParkaNet pnet, BrowserWin browser)
    {
	predName = null;
	domName = null;
	ranName = null;
	query = null;
	pn = pnet;
	bwin = browser;
	errMsg = "";
    }

    PredicateData (String linkname, ParkaNet pnet, BrowserWin browser)
    {
	predName = linkname;
	domName = null;
	ranName = null;
	query = null;
	pn = pnet;
	bwin = browser;
	errMsg = "";
    }

    // need to call this routine before calling 'sendQuery()' if first
    // constructor was used.
    //
    void setLinkName (String name)
    {
	predName = name;
    }

    void setDomainValue (String name)
    {
	domName = name;
    }

    void setRangeValue (String name)
    {
	ranName = name;
    }

    // 'query' is filled if predName exists;
    // otherwise it is left null.
    //
    public void run()
    {
	if (predName != null)
	{
	    bwin.setCursor (Frame.WAIT_CURSOR);

	    for (int i = 0; i < builtinQueryOnlyPreds.length; i++)
	    {
		if (predName.equals (builtinQueryOnlyPreds[i]))
		{
	            errMsg = "This predicate is used for queries only;  "
                            + "it does not exist in any assertion used "
                            + "to build a KB.";
		    bwin.displayForBrowser (errMsg);
		    return;
		}
	    }
/*          No longer needed.
	    for (int i = 0; i < builtinStructPreds.length; i++)
            {
		if (predName.equals (builtinStructPreds[i]))
                {
		    if ((domName == null) && (ranName == null))
		    {
                        //when using sampling for limiting data sent back
                        //from the server side, we don't need special
			//handling here.
	                errMsg = "Need a domain or a range to limit output";
		        bwin.displayForBrowser (errMsg);
		        return;
		    }
		}
		// if structural and have a dom or ran, handle below
            }
*/
	    if ((domName == null) && (ranName == null))
	        query = String.valueOf(FRAME_UNRESTRICT) + " " +
                          String.valueOf(BrowserWin.PREDCONST) + " "
                             + predName + "\n";
            else    if ((domName == null) && (ranName != null))
		query = String.valueOf(FRAME_RESTRICT_IN) + " " +
                       String.valueOf(BrowserWin.PREDCONST) + " " + predName
                       + "\n" + String.valueOf(BrowserWin.FRAMECONST)
                       + " " + ranName + "\n";
            else    if ((domName != null) && (ranName == null))
		query = String.valueOf(FRAME_RESTRICT_OUT) + " " +
                       String.valueOf(BrowserWin.PREDCONST) + " " + predName
                       + "\n" + String.valueOf(BrowserWin.FRAMECONST)
                       + " " + domName + "\n";
            else    //neither is null
		query = String.valueOf(DATAVERIFY) + " " +
                       String.valueOf(BrowserWin.PREDCONST) + " " + predName
                       + "\n" + String.valueOf(BrowserWin.FRAMECONST)
                       + " " + domName + "\n" + 
                       String.valueOf(BrowserWin.FRAMECONST) + " "
                       + ranName + "\n";

	    int  status = pn.submitBrowsingQuery (query);
	    errMsg = pn.getBrowserMsg();

            if (status >= -1)    // 0 is no error, all data;
                                 // -1 is no error, sample of data
	    {
	        // Note that a predicate must have a before and after node,
	        // unlike the case with nodes-- they may or may not have
	        // both a before and after predicate.   Nodes might have
	        // only one of those.  This means we do not have to handle
	        // any special cases.
	        //
		String[][]  answerLists = pn.getBrowserAnswers();

		if (answerLists != null)
		{
		    for (int i = 0; i < answerLists.length; i++)
		    {
		        String[]  column = answerLists[i];
		        bwin.fillList (column);
	            }
	        }
		else
		{
		// This seems to always be programmer error when it happens;
		// but we will leave it in, in case.
		//
		    errMsg = "Empty answer list";
		    bwin.displayForBrowser (errMsg);
// TEST
System.out.println ("This is probably a programmer error.  Check it out.");
		}
	    }
	    if (errMsg.length() > 0)
		bwin.displayForBrowser (errMsg);

	    bwin.setCursor (Frame.DEFAULT_CURSOR);
	}
    }
}
