/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */

package  parka;

import   java.awt.*;
import   java.util.*;
import   java.applet.*;
import   java.net.*;

/* For managing displayed results of a database query
 */
public class QueryResults  extends Panel  implements MultiColumnCallback
{
    QueryWin      parentWin;
    MultiColumn   resultList;
    Thread        resultThread;
    ParkaNet      pn;
    DrawArea      da;

    static private String   START_LBL_STR = "First Item: ";
    static private String   END_LBL_STR = "Last Item: ";
    static private String   NEXT_CB_STR = "Get Next Group";
    static private String   PREV_CB_STR = "Get Last Group";
    static private int      NUMCOLS_TEXTF = 10;
    static private int      GRIDROWS = 2;
    static private int      GRIDCOLS = 2;
    static private int      HORGAP = 2;
    static private int      VERTGAP = 2;

    Panel         outputIndicators;
    TextField     startText;
    TextField     endText;
    Checkbox      nextCB;
    Checkbox      prevCB;
    CheckboxGroup cg;

    int           totalNumAnswers;
    int           numCols;
    int           firstMultiListItem;
    int           lastMultiListItem;
    static private int    blockSize; //number of answers obtained per call
                                     //to server

  AppletContext context;    // used to load documents in web-browser

    static final String   NOVARSINQUERY = "No variables retrieved";
	//MUST match value returned from server when request for
	//variables is given.

    QueryResults (QueryWin parWin, ParkaNet netConnect, DrawArea querySrc)
    {
        super();

        parentWin = parWin;
        resultList = null;
        resultThread = null;
	pn = netConnect;
	da = querySrc;
	blockSize = 100;      //arbitrary choice, but small enough not
                              //to upset Vectors inside MultiColumn.
        setupIndicatorPanel();
	initIndicators(); //do after 'setupIndicatorPanel()'
        setLayout (new BorderLayout (0, 0));

	/* Heflin 5/9: Code for applet context. Use to load html documents */
	Container grandparent;

	/* determine if this is in an Applet */
	grandparent = parentWin.parent;
	if (grandparent instanceof Applet) 
	  context = ((Applet)grandparent).getAppletContext();
	else
	  context = null;

    }

    private void initIndicators()
    {
	firstMultiListItem = 0;
	lastMultiListItem = 0;
	numCols = 0;
	totalNumAnswers = 0;
	setTextFields();
        prevCB.disable();  //must have already created.
        nextCB.disable();  //must have already created.
	pn.clearNumAnswersToQueryMsg();
    }

    private void setupIndicatorPanel()
    {
	outputIndicators = new Panel();
	outputIndicators.setLayout (new GridLayout(GRIDROWS, GRIDCOLS,
                        HORGAP, VERTGAP));
	Label  start = new Label (START_LBL_STR, Label.LEFT);
	Label  end = new Label (END_LBL_STR, Label.LEFT);
	startText = new TextField (NUMCOLS_TEXTF);
	startText.setEditable (false);
	endText = new TextField (NUMCOLS_TEXTF);
	endText.setEditable (false);
	Panel  txtIndStart = new Panel();  //FlowLayout is default
	txtIndStart.add (start);
	txtIndStart.add (startText);
	Panel  txtIndEnd = new Panel();    //FlowLayout is default
	txtIndEnd.add (end);
	txtIndEnd.add (endText);

	nextCB = new Checkbox (NEXT_CB_STR);
	prevCB = new Checkbox (PREV_CB_STR);
	cg = new CheckboxGroup();
	prevCB.setCheckboxGroup(cg);
	nextCB.setCheckboxGroup(cg);
	cg.setCurrent (nextCB);

        outputIndicators.add (txtIndStart);
        outputIndicators.add (txtIndEnd);
        outputIndicators.add (prevCB);
        outputIndicators.add (nextCB);
    }

    private void setTextFields()
    {
	startText.setText (String.valueOf(firstMultiListItem));
	endText.setText (String.valueOf(lastMultiListItem));
    }

    void clear()
    {
	initIndicators();
	if (resultList != null)
	{
	    resultList.clear();
	    resultList.erase_titles();
	}
    }

    public boolean action (Event e, Object ob)
    {
	if (e.target  instanceof  Checkbox)
	{
	    Checkbox cb = (Checkbox)e.target;
	    if ( (cb.getLabel()).equals (NEXT_CB_STR))
	    {
		firstMultiListItem += blockSize;
		lastMultiListItem += blockSize;
		if (lastMultiListItem >= totalNumAnswers)
		{
		    lastMultiListItem = totalNumAnswers;
		    nextCB.disable();
		}
		setTextFields ();
		if (!prevCB.isEnabled())
		    prevCB.enable();
		putAnswersInMultiList();
		return   true;
	    }
	    else  if ( (cb.getLabel()).equals (PREV_CB_STR))
	    {
		lastMultiListItem -= blockSize;
		firstMultiListItem = lastMultiListItem - blockSize;
		if (firstMultiListItem < 1)
		{
		    firstMultiListItem = 1;
		    prevCB.disable();
		}
                //readjust, to make sure we have a whole blocksize of answers:
		if (firstMultiListItem == 1)
		    lastMultiListItem = blockSize;

		setTextFields ();
		if (!nextCB.isEnabled())
		    nextCB.enable();
		putAnswersInMultiList();
		return   true;
	    }
	    else
	        System.out.println ("Programmer Error -- " +
                          "Cannot recognize checkbox used");
	}
	return   false;
    }

    void doQuery()
    {
        pn.submitQuery (da.links);
	    //valid to call various query related data calls in ParkaNet
	    //class AFTER this call.

        // If no results (pn.vars is null) then don't show results
        if (pn.getVariableList() == null)
        {
            resultList=null;
            removeAll();   //delete answers from an old query
	    initIndicators();
            validate();
        }
        else
        {
            resultList = new MultiColumn (pn.getVariableList(), this);
		//variable list provides column headers
            removeAll();
            add("North", outputIndicators);
            add("Center", resultList);
            validate();

            initAnswersForMultiList();
        }
    }

    private  void initAnswersForMultiList()
    {
	totalNumAnswers = pn.getNumTotalAnswersToQuery();
        numCols = pn.getNumVariables();

	if (prevCB.isEnabled())
	    prevCB.disable();

	if (totalNumAnswers > blockSize)
	    nextCB.enable();
	else if (nextCB.isEnabled())
	    nextCB.disable();

	// call for first set of answers for the user.
	firstMultiListItem = 1;
        lastMultiListItem = Math.min (blockSize, totalNumAnswers);
	setTextFields();
	putAnswersInMultiList();
        validate();
    }

    //Must be called sometime after pn.submitQuery
    //and after firstMultiListItem and lastMultiListItem
    //are correctly set.
    //
    private  void putAnswersInMultiList()
    {
	// If we do not call 'MultiColumn.clear()', answers just
	// accumulate.  Of course that would require different handling
	// than we do here, since when a user would go back and forward
	// multiple times with the current way of doing things, we'd
	// just accumulate more and more answers (same ones appending).
	//
	resultList.clear();

        pn.getAnswersFromServer (firstMultiListItem,
                           lastMultiListItem * numCols);

        String[] answers = pn.getAnswers();
	/* TEST
	for (int k = 0; k < answers.length; k++)
	    System.out.println (answers[k]);
	*/
	int numRows = (lastMultiListItem - firstMultiListItem) + 1;
	for (int i = 0; i < numRows; i++)
	{
	    String[]  oneRow = new String[numCols];
	    for (int j = 0; j < numCols; j++)
	    {
		oneRow[j] = answers[(i*numCols)+j];
            }
	    resultList.addItem (oneRow);
	}
	validate();
	parentWin.validate();
    }

    /* Called on a single click on a list item.
       The selected list items are: 0 for the first item,
       1 for the second, etc, thru n-1 for the nth.
    */
    public void singleClick (MultiColumn list, int row, int col)
    {
        // clear all bindings
        da.unbindVariables();

        if (row >= 0)   //valid list item
        {
            // set bindings
            String[]  binds = list.getItem(row);
            for (int i = 0; i < list.title.length; i++) 
            {
                Icon  matching_var = da.getVariable (list.title[i]);
		if (matching_var != null)
		{
                    matching_var.changeBinding (binds[i]);
		}
            }
        }
        da.paintDrawArea();
    }

    /* Called upon double-clicking on a list item.
    */

  /* Heflin 5/8: Tried changing this so that applets would load a document
     into the browser based on the user's click. */

    public void doubleClick(MultiColumn list, int row, int col)
    {
      /*
	// Use double click to clear the effect of single clicking
        // on the list.
	//
	singleClick (list, -1);
	*/

      URL url = null;

      if (context != null) {                  // only works for applets
	/* retrieve bindings  */
	String[] binds = list.getItem(row);
	if (binds[col].startsWith("http://")) {         // probably a URL
	  try {
	    url = new URL(binds[col]);
	    System.out.println("Trying to show " + url);
	    // display document in a new window named "SHOE_Doc"
	    context.showDocument(url, "SHOE_Doc");
	  } catch (MalformedURLException ex) {
	    System.out.println(url + " not valid"); 
	  }
	}

	/*
	boolean done = false;
	int i = 0;
	while (i < list.title.length && done == false) {
	  if (binds[i].startsWith("http://")) {         // probably a URL
	    try {
	      url = new URL(binds[i]);
	      System.out.println("Trying to show " + url);
	      // display document in a new window named "SHOE_Doc"
	      context.showDocument(url, "SHOE_Doc");
	      done = true;
	    } catch (MalformedURLException ex) {
	      System.out.println(url + " not valid"); 
	      // simply try next field
	    }
	  }
         i = i+1;  
	}
	*/

      }
    }  
	    
}
