/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package parka;

import java.awt.*;
import java.util.*;

/**********************************************************************
  window
**********************************************************************/

public class QueryWin extends ParkaWin
{
    // main components
    ToolBar tb;
    SplitPanel sp;
    DrawArea da;
    QueryResults resultArea;

    // network interface
    ParkaNet pn;

    // browser window
    BrowserWin  browser;

    // applet or standalone application pointer
    Container parent;

    // fonts
    String fontname;
    int fontsize;

    // menu stuff
    MenuBar mb;
    Menu FileMenu;
    Menu FontMenu;
    Menu ParkaMenu;
    Menu ObjectMenu;
    Menu LinkMenu;

    static final String FILE_MENU_NAME = "File";
    static final String PARKA_MENU_NAME = "Parka";
    static final String OBJECT_MENU_NAME = "Object";
    static final String LINK_MENU_NAME = "Link";
    static final String FONT_MENU_NAME = "Font";
    static String[]   fileMenuItems = {"Close", "New Object", "Clear Graph",
                                "Clear Query Results", "Browser Window",
                                "Application Window" };
    static int[]  fileMenuSeps = {4};
    static String[]   parkaMenuItems = {"Connect to Parka",
                 "Open Database", "Submit Query", "Open Browser",
                 "Disconnect from Parka", "Close Database"
                  /* , "Debug Print" */ };
    static int[]  parkaMenuSeps = {4 /* , 6 */ };
    static final int  BROWSWIN_MENUITEM_LOC = 3;
                         //change if change parkaMenuItems
    static String[]   linkMenuItems = {"Reverse Link", "Rename Link",
                 "Delete Link"};
    static String[]   objectMenuItems = {"Rename Object",
                 "Delete Object"}; //old 1st ob "New Object" no longer in array
    static int[]  objectMenuSeps = {2};  //an item will be added after this
    static String[]   fontSizes = {"9", "10", "12", "14", "16", "18", "24"};
    static int        defaultFSize = 2; //position of default size in array
    static String[]   fontType = {"Courier", "Helvetica", "Times"};
    static int        defaultFType = 1; //position of default type

    CheckboxMenuItem isvariableMenuItem;
    CheckboxMenuItemGroup cg1, cg2;

    /*  ***********************************************  */

    // Constructor from applet:
    //
    public QueryWin (Parka MyApplet, int width, int height)
    {
        setup_win (MyApplet, width, height);
	browser = null;
    }

    public QueryWin (Parka MyApplet, int width, int height,
		     String parkaHost, int parkaPort, String parkaKB)
    {
        setup_win (MyApplet, width, height);
	browser = null;
	
	/* Heflin: 5/21/98 */
	if (parkaHost != null)
	  pn.changeParkaHost(parkaHost, parkaPort);
	if (parkaKB != null)
	  pn.initLoadKBbyName(parkaKB, true);
    }

        // Constructor from standalone application:
    public QueryWin (ParkaApp MyApp, int width, int height)
    {
        setup_win (MyApp, width, height);
	browser = null;
    }

    void setup_win (Container cont, int width, int height)
    {
        //Font Info
        fontname = fontType[defaultFType];
        fontsize = Integer.parseInt (fontSizes[defaultFSize]);

        setTitle("Parka Interface v. 1.0");
        parent = cont;
        mb = new MenuBar();
        pn = new ParkaNet(this);
        tb = new ToolBar(this);
        da = new DrawArea (this, tb);
        tb.setDrawArea (da);

        // be sure the ParkaNet and DrawArea constructors have
        // already been called
        resultArea = new QueryResults (this, pn, da);

        ScrollbarPanel p = new ScrollbarPanel (da);

        sp = new SplitPanel(SplitPanel.VERTICAL);
        sp.add (p, 0.9);
        sp.last (resultArea, 0.1);
        setLayout (new BorderLayout(0,0)); // gap 0 pixels
        add ("North", tb);
        add ("Center", sp);
        resize (width, height);

        // Menu bar
        setMenuBar(mb);
        
    // File Menu
        MenuMaker  makeFileM;

        makeFileM = new MenuMaker (FILE_MENU_NAME,
                                MenuMaker.SIMPLE, fileMenuItems, fileMenuSeps);
        FileMenu = makeFileM.createMenu();
        //NOTE:  Change index/file ref if move menu item for raising the
        //       application window.
	if (ApplType.getIsApplet())
	{          // make this menu item insensitive:
	    MenuItem  mi = FileMenu.getItem (6);
	    mi.disable();
	}
        mb.add(FileMenu);

    // Parka Menu
        makeFileM = new MenuMaker (PARKA_MENU_NAME,
                            MenuMaker.SIMPLE, parkaMenuItems, parkaMenuSeps);
        ParkaMenu = makeFileM.createMenu();
        mb.add(ParkaMenu);

    // Object Menu
        makeFileM = new MenuMaker (OBJECT_MENU_NAME,
                             MenuMaker.SIMPLE, objectMenuItems, objectMenuSeps);
        ObjectMenu = makeFileM.createMenu();
        mb.add(ObjectMenu);
        //create a mixed menu type with the following addition:
        isvariableMenuItem = new CheckboxMenuItem("Variable");
        ObjectMenu.add (isvariableMenuItem);
        ObjectMenu.disable();

    // Link Menu
        makeFileM = new MenuMaker (LINK_MENU_NAME,
                             MenuMaker.SIMPLE, linkMenuItems, null);
        LinkMenu = makeFileM.createMenu();
        mb.add(LinkMenu);
        LinkMenu.disable();

    // Font Menu
        CheckboxMenuItem defmen;
        cg1 = new CheckboxMenuItemGroup();
        cg2 = new CheckboxMenuItemGroup();
        defmen = cg1.add (fontSizes, defaultFSize);
        cg1.set(defmen);
        defmen = cg2.add (fontType, defaultFType);
        cg2.set(defmen);

        Vector[]  cmiGroupInfo = new Vector[2];
        cmiGroupInfo[0] = cg1.getCheckboxMIGroup();
        cmiGroupInfo[1] = cg2.getCheckboxMIGroup();

        makeFileM = new MenuMaker (FONT_MENU_NAME, cmiGroupInfo);
        FontMenu = makeFileM.createMenu();
        mb.add(FontMenu);

        pack();
	validate();
    }

    void  openBrowserWin()
    {
	if (browser == null)
	{
            browser = new BrowserWin (pn, this);
            browser.show();

	    MenuItem  mi = ParkaMenu.getItem (BROWSWIN_MENUITEM_LOC);
	    mi.disable();
	}
    }

    void  closeBrowserWin()
    {
	browser.dispose();
	    //since browser uses the ParkaNet assoc. with this window,
	    //we will close browser too.  QueryWin currently is the place
	    //that ParkaNet tries to display all its status messages.
            //In addition, while the ParkaNet object should still
	    //be valid, opening a new QueryWin means a new server
	    //connection; we currently can't handle more than one to the
	    //server, and there would have to be restrictions on multiple
	    //users in one KB.
	browser = null;

	MenuItem  mi = ParkaMenu.getItem (BROWSWIN_MENUITEM_LOC);
	mi.enable();
    }

    public void closeWin()
    {
	if (browser != null)
	    closeBrowserWin();

        if (ApplType.getIsApplet())
            ((Parka)parent).WinV.removeElement(this);
        else  if (ApplType.getIsApplication())
            ((ParkaApp)parent).WinV.removeElement(this);
	hide();
        dispose();
    }

    void  reinitWindow()
    {
	da.clearGraph();
	resultArea.clear();
    }

    // when: if a KB was open and is exited.
    void  reinitBrowser()
    {
	if (browser != null)
	    browser.oldKBclosed();
    }

    void  reinitAppWin()
    {
	reinitWindow();
	reinitBrowser();
    }

    void  redrawGraphics()
    {
	da.paintDrawArea();
    }

    public Dimension minimumSize()
    {
         int MIN_WIDTH = 150, MIN_HEIGHT = 200;
         return new Dimension(MIN_WIDTH, MIN_HEIGHT);
    }   

    public Dimension preferredSize()
    {
        int PREF_WIDTH = 400, PREF_HEIGHT = 400;
        return new Dimension(PREF_WIDTH,PREF_HEIGHT);
    }

    public boolean action (Event evt, Object arg)
    {
        boolean   result = true;   //assume handled

        if (evt.target instanceof CheckboxMenuItem)
        {
            // Save old cursor and display wait cursor
            int oldCursor = this.getCursorType();
            this.setCursor(Frame.WAIT_CURSOR);

	    // 'arg' (CheckboxMenuItem new state) is not needed
            String  label = (String)
                        ((CheckboxMenuItem)evt.target).getLabel();

            if ((label.equals("Variable")) && (da.active instanceof Icon))
            {
                da.changeVariable(isvariableMenuItem.getState(), false);
            }
            else if (label.equals(fontSizes[0]))
            {
                changeSize (Integer.parseInt (fontSizes[0]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[1]))
            {
                changeSize (Integer.parseInt (fontSizes[1]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[2]))
            {
                changeSize (Integer.parseInt (fontSizes[2]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[3]))
            {
                changeSize (Integer.parseInt (fontSizes[3]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[4]))
            {
                changeSize (Integer.parseInt (fontSizes[4]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[5]))
            {
                changeSize (Integer.parseInt (fontSizes[5]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontSizes[6]))
            {
                changeSize (Integer.parseInt (fontSizes[6]));
                cg1.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontType[0]))
            {
                changeFont (fontType[0]);
                cg2.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontType[1]))
            {
                changeFont (fontType[1]);
                cg2.set((CheckboxMenuItem)(evt.target));
            }
            else if (label.equals(fontType[2]))
            {
                changeFont (fontType[2]);
                cg2.set((CheckboxMenuItem)(evt.target));
            }
        }
        else  if (evt.target instanceof MenuItem)
        {
            // Save old cursor and display wait cursor
            int oldCursor = this.getCursorType();
            this.setCursor(Frame.WAIT_CURSOR);

            String label = (String) arg;
            
            if (label.equals (fileMenuItems[0]))
            {
                pn.disconnect();
                closeWin();
            }
            else if (label.equals (fileMenuItems[1]))
            {
                // new object with default location
                da.createIcon (DrawArea.DEFAULT_NEWICON_X,
                                  DrawArea.DEFAULT_NEWICON_Y);
            }
            else if (label.equals (fileMenuItems[2]))
            {
		da.clearGraph();
            }
            else if (label.equals (fileMenuItems[3]))
            {
		resultArea.clear();
            }
            else if (label.equals (fileMenuItems[4]))
            {
		if (browser != null)
		    browser.toFront();
            }
            else if (label.equals (fileMenuItems[5]))
            {
		appWinToFront();
            }
            else if (label.equals (parkaMenuItems[0]))
            {
                pn.doPortDialog();
            }
            else if (label.equals (parkaMenuItems[1])) //open KB
            {
                pn.initKBLoading();
            }
            else if (label.equals (parkaMenuItems[2]))
            {
                da.unbindVariables(); // Cleanup before doing query
                resultArea.doQuery();
            }
            else if (label.equals (parkaMenuItems[3]))
            {
		openBrowserWin();
            }
            else if (label.equals (parkaMenuItems[4]))
            {   
                pn.disconnect();
            }
            else if (label.equals (parkaMenuItems[5]))
	    {
                pn.closeKB();
	    }
/*
            else if (label.equals (parkaMenuItems[6]))
            {     
                da.printDebug();
            }
*/
            else if ((label.equals (objectMenuItems[1])) &&
                                (da.active instanceof Icon))
            {
                da.deleteObject();
            }
            else if ((label.equals (objectMenuItems[0])) &&
                                (da.active instanceof Icon))
            {
                da.renameObject();
            }
            else if ((label.equals (linkMenuItems[0])) &&
                                 (da.active instanceof Link))
            {
                da.reverseLink();
            }
            else if ((label.equals (linkMenuItems[1])) &&
                                     (da.active instanceof Link))
            {
		da.renameObject();
            }
            else if ((label.equals (linkMenuItems[2])) &&
                                      (da.active instanceof Link))
            {
                da.deleteObject();
            }

            // Restore old cursor
            this.setCursor(oldCursor);
        }
        else
        result = false;

        return   result;
    }

    // put orig application window on top of window stack:
    //
    void appWinToFront()
    {
	if (ApplType.getIsApplication())
	    ((Window)parent).toFront();
    }

    void setMenuSensitivity (boolean objMenuNewState, boolean linkMenuNewState,
                             boolean stateOfActiveItem)
	//3rd parameter is only relevant for Object menus.
    {
	if (objMenuNewState == true)
        {
	    ObjectMenu.enable();
	    isvariableMenuItem.setState (stateOfActiveItem);
        }
	else
	    ObjectMenu.disable();
	if (linkMenuNewState == true)
	    LinkMenu.enable();
        else
	    LinkMenu.disable();
    }

    void  reset_font (String fname, int size)
    {
        Font f = new Font(fname, Font.PLAIN, size);
        FontMetrics fm;
        if (ApplType.getIsApplet())
            fm = ((Parka)parent).getFontMetrics(f);
        else  if (ApplType.getIsApplication())
            fm = ((ParkaApp)parent).getFontMetrics(f);
        else
            fm = null;

        if (fm != null)
            da.updateFont(f, fm);      
    }

    public void changeFont(String newfont)
    {
        fontname = newfont;
        reset_font (fontname, fontsize);
    }

    public void changeSize(int size)
    {
        fontsize = size;
        reset_font (fontname, fontsize);
    }

    public boolean handleEvent(Event evt)
    {
        if (evt.id == Event.WINDOW_DESTROY)
        {
            closeWin();
            return true;
        }
        else
            return super.handleEvent(evt);
    }

    public void displayMessage(String msg)
    {
        // Unselects the current object and
        // displays a message in the toolbar.
        //
        da.selectObject(null);
        tb.displayMessage(msg);
    }

    // display message, keep state.
    public void displayMessageKS(String msg)
    {
        // displays a message in the toolbar,
	// leaving the current activation state
	// in DrawArea unaffected.
        //
        tb.displayMessage(msg);
    }
}

/* Simple private class that handles the scroll bars
*/
class ScrollbarPanel extends Panel
{
    DrawArea d;   //DrawArea already has two scrollbars in it

    ScrollbarPanel(DrawArea da)
    {
        d = da;
	setLayout(new BorderLayout());
        add ("Center", da);
    }

    public boolean handleEvent (Event evt)
    {
        if (evt.target instanceof Scrollbar)
        {
	//NOTE:  names of events -- all changed in JDK 1.1-- see
	//Scrollbar class

            if (evt.id== Event.SCROLL_LINE_UP || 
                evt.id==  Event.SCROLL_LINE_DOWN ||
                evt.id==  Event.SCROLL_PAGE_UP ||
                evt.id==  Event.SCROLL_PAGE_DOWN ||
                evt.id==  Event.SCROLL_ABSOLUTE)    
            {
                    d.translate();
                    return true;
            }
        }
        return false;
    }
}
