/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package parka;

import java.awt.*;
import java.util.*;

public class SplitPanel extends Panel
{
    public int type;
    public SplitSubPanel sub;
    public SplitBar bar;
    
    static final float DEFAULT_WEIGHT = (float)(0.1);
    static final int HORIZONTAL = 0;
    static final int VERTICAL = 1;

    public SplitPanel(int _type)
    {
        type = _type;
        setLayout (new BorderLayout());
        sub = new SplitSubPanel(this);
        bar = new SplitBar(this);
        add ("Center",sub);
        if (_type == HORIZONTAL)
            super.add("North",bar);
        else
            super.add("West", bar);
    }

    private void layout_settings (GridBagConstraints gbc, double weight)
    {
        if (type == HORIZONTAL)
        {
            gbc.weightx = weight;
            gbc.weighty = 1;
            gbc.gridx = GridBagConstraints.RELATIVE;
            gbc.gridy = 1;
        }
        else
        {
            gbc.weightx = 1;
            gbc.weighty = weight;
            gbc.gridx = 1;
            gbc.gridy = GridBagConstraints.RELATIVE;
        }
    }

    public Component add (Component comp, double weight)
    {
        layout_settings (sub.gbc, weight);

        sub.gbl.setConstraints (comp, sub.gbc);
        sub.add (comp);
        bar.add (comp);

        return   comp;
    }

    public Component last (Component comp, double weight)
    {
        layout_settings (sub.gbc, weight);

        if (type == HORIZONTAL)
        {
            sub.gbc.gridwidth = GridBagConstraints.REMAINDER;
        }
        else
        {
            sub.gbc.gridheight = GridBagConstraints.REMAINDER;
        }
        sub.gbl.setConstraints (comp, sub.gbc);
        sub.add (comp);
        bar.add (comp);

        return   comp;
    }
}

class SplitBar extends Canvas
{
    static final int SPLIT = 14;            //   /4

    Vector components;
    public SplitPanel sp;
    int dx;
    int dy;
    int lastx;
    int lasty;
    int firstx;
    int firsty;
    boolean dragging=false;
    boolean outside=false;
    
    Component currentc;

    public SplitBar(SplitPanel s)
    {
        sp=s;
        components=new Vector();    
    }
    
    public void paint(Graphics g)
    {
        /* Draw background*/
        Rectangle b= bounds();
        g.setColor(getBackground());
        g.fillRect(0,0,b.width,b.height);
        
        /* Draw Foreground */
        g.setColor(Color.darkGray);
        if (sp.type==SplitPanel.HORIZONTAL)
        {
            g.fillRect(0,0,b.width,b.height/2);
            g.setColor(Color.black);
            g.drawLine(0,b.height/2,b.width,b.height/2);
        }
        else
        {
            g.fillRect(0,0,b.width/2,b.height);
            g.setColor(Color.black);
            g.drawLine(b.width/2,0,b.width/2,b.height);
        }
        
        /* Draw Rects */
        g.setColor(Color.white);
        Enumeration e=components.elements();
        if (e.hasMoreElements())
            e.nextElement();    // get rid of first...
        while(e.hasMoreElements())
        {
            Point location = ((Component)(e.nextElement())).location();
            if (sp.type==SplitPanel.HORIZONTAL)
            {
                  g.fill3DRect(location.x-SPLIT/2,0,SPLIT,SPLIT,true);
            }
            else
            {
                g.fill3DRect(0,location.y-SPLIT/2,SPLIT,SPLIT,true);
            }
        }
        
    }

    public Dimension minimumSize()
    {
        return new Dimension(SPLIT,SPLIT);
    }
        
    public Dimension preferredSize()
    {
        return minimumSize();
    }

    public boolean hit(Point location, int x, int y)
    {
        Rectangle r;
        if (sp.type==SplitPanel.HORIZONTAL)
        {
            r=new Rectangle(location.x-SPLIT/2,0,SPLIT,SPLIT);
        }
        else 
        {
            r=new Rectangle(0,location.y-SPLIT/2,SPLIT,SPLIT);
        }
        return r.inside(x,y);
    }

    public boolean mouseDown(Event evt, int x, int y)
    {
    /* Determine who got hit */
        
        dragging=true;
        outside=false;
        currentc=null;
        Component c;
        Point location=new Point(0,0);
        Enumeration e=components.elements();
        if (e.hasMoreElements())
            e.nextElement();    // get rid of first...
        while(e.hasMoreElements())
        {
            location = (c=(Component)(e.nextElement())).location();
            if (hit(location,x,y))
            {
                currentc = c;
                break;
            }
        }
        
        if (currentc == null)
                    return false;
        
        int drawx, drawy;
        
        if (sp.type == SplitPanel.HORIZONTAL)
        {
            drawx = location.x;
            drawy = 0;
        }
        else
        {
            drawx = 0;
            drawy = location.y;
        }
        dx = drawx - x;
        dy = drawy - y;
        
        Graphics g = getGraphics();
        g.setXORMode (Color.black);
        g.setColor (Color.white);
        if (sp.type == SplitPanel.HORIZONTAL)
        {
            g.fillRect (drawx - SPLIT/2,0,SPLIT,SPLIT);
        }
        else
        {
            g.fillRect(0,drawy - SPLIT/2,SPLIT,SPLIT);
        }
        firstx = drawx;
        firsty = drawy;
        lastx=drawx;
        lasty=drawy;
        return true;
    }
        
    public boolean mouseDrag(Event evt, int x, int y)
    {
        if (currentc==null) return false;
        if (!dragging || outside) return false; 
        int drawx,drawy;
        
        drawx=x+dx;
        drawy=y+dy;
        
        Rectangle b=bounds();   
        Graphics g=getGraphics();
        g.setXORMode(Color.black);
        g.setColor(Color.white);
        if (sp.type==SplitPanel.HORIZONTAL)
            {
            drawy=0;
            if (drawx<b.x) drawx=b.x;
            if (drawx>b.x+b.width) drawx=b.x+b.width;
            g.fillRect(lastx-SPLIT/2,drawy,SPLIT,SPLIT);
            g.fillRect(drawx-SPLIT/2,drawy,SPLIT,SPLIT);
            }
        else
            {
            drawx=0;
            if (drawy<b.y) drawy=b.y;
            if (drawy>b.y+b.height) drawy=b.y+b.height;
            g.fillRect(drawx,lasty-SPLIT/2,SPLIT,SPLIT);
            g.fillRect(drawx,drawy-SPLIT/2,SPLIT,SPLIT);
            }
        lastx=drawx;
        lasty=drawy;
        return true;
    }
        
    public boolean mouseExit(Event evt, int x, int y)
    {
        outside=true;
        if (currentc==null) return false;
        if (!dragging) return false;    
        int drawx,drawy;
        
        Rectangle b=bounds();   
        Graphics g=getGraphics();
        g.setXORMode(Color.black);
        g.setColor(Color.white);
        if (sp.type==SplitPanel.HORIZONTAL)
            {
            g.fillRect(lastx-SPLIT/2,0,SPLIT,SPLIT);
            }
        else
            {
            g.fillRect(0,lasty-SPLIT/2,SPLIT,SPLIT);
            }
        lastx=firstx;
        lasty=firsty;
        
        return true;
    }
    
    public boolean mouseEnter(Event evt, int x, int y)
    {
        outside=false;
        if (currentc==null) return false;
        if (!dragging) return false;    
        int drawx,drawy;
        
        drawx=x+dx;
        drawy=y+dy;
                
        Rectangle b=bounds();   
        Graphics g=getGraphics();
        g.setXORMode(Color.black);
        g.setColor(Color.white);
        if (sp.type==SplitPanel.HORIZONTAL)
        {
            drawy=0;
            if (drawx<b.x) drawx=b.x;
            if (drawx>b.x+b.width) drawx=b.x+b.width;
            g.fillRect(drawx-SPLIT/2,drawy,SPLIT,SPLIT);
        }
        else
        {
            drawx=0;
            if (drawy<b.y) drawy=b.y;
            if (drawy>b.y+b.height) drawy=b.y+b.height;
            g.fillRect(drawx,drawy-SPLIT/2,SPLIT,SPLIT);
        }
        lastx=drawx;
        lasty=drawy;

        return true;
    }

    public boolean mouseUp(Event evt, int x, int y)
    {
        if (currentc==null) return false;   
        if (!dragging) return false;    
        dragging=false;
        int drawx,drawy;
        
        drawx=x+dx;
        drawy=y+dy;
        
        Rectangle b = bounds();
        Graphics g=getGraphics();
        GridBagLayout gbl=(GridBagLayout)(sp.sub.getLayout());
        
        g.setXORMode(Color.black);
        g.setColor(Color.white);
        if (sp.type==SplitPanel.HORIZONTAL)
        {
            drawy=0;
            if (drawx<b.x) drawx=b.x;
            if (drawx>b.x+b.width) drawx=b.x+b.width;
            g.fillRect(lastx-SPLIT/2,0,SPLIT,SPLIT);
                        
            /* Find before and after values */
            int divisor= components.indexOf(currentc);
                             // everything up to and including divisor is before
            Enumeration e=components.elements();
            while(e.hasMoreElements())
            {
                Component c=(Component)(e.nextElement());
                GridBagConstraints gbc=gbl.getConstraints(c);
                if (components.indexOf(c) < divisor)
                {
                    /* Before */
                    //System.out.println("Component " + c.toString() + " BEFORE");
                    gbc.weightx = drawx-(c.minimumSize().width);
                }
                else
                {
                    /* After */
                    //System.out.println("Component " + c.toString() + " AFTER");
                    gbc.weightx = b.x+b.width-drawx-(c.minimumSize().width);
                    }
                if (gbc.weightx<0) gbc.weightx=0;
                gbl.setConstraints(c,gbc);
                //System.out.println("New: " + gbc.toString());
            }
        }
        else
        {
            drawx=0;
            if (drawy<b.y) drawy=b.y;
            if (drawy>b.y+b.height) drawy=b.y+b.height;
            g.fillRect(0,lasty-SPLIT/2,SPLIT,SPLIT);
            
            /* Find before and after values */
            int divisor= components.indexOf(currentc);
                            // everything up to and including divisor is before
            Enumeration e=components.elements();
            while(e.hasMoreElements())
            {
                Component c=(Component)(e.nextElement());
                GridBagConstraints gbc=gbl.getConstraints(c);
                if (components.indexOf(c) < divisor)
                {
                    /* Before */
                    gbc.weighty = drawy-(c.minimumSize().height);
                }
                else
                {
                    /* After */
                    gbc.weighty = b.y+b.height-drawy-(c.minimumSize().height);
                }
                if (gbc.weighty<0) gbc.weighty=0;
                gbl.setConstraints(c,gbc);
            }

        }
        sp.sub.layout();
        sp.bar.paintAll(sp.bar.getGraphics());
        sp.sub.paintAll(sp.sub.getGraphics());      // needed?
        sp.sub.paintComponents(sp.sub.getGraphics());   // needed?
        return true;
    }

    public void add(Component c)
    {
        components.addElement(c);
    }
}

class SplitSubPanel extends Panel
{
    public GridBagLayout gbl;
    public GridBagConstraints gbc;
    public SplitPanel sp;
    
    public SplitSubPanel(SplitPanel s)
    {
        gbl= new GridBagLayout();
        setLayout(gbl);
        gbc=new GridBagConstraints();
        gbc.fill=GridBagConstraints.BOTH;
        sp=s;
        
        if (sp.type==SplitPanel.HORIZONTAL)
        {
            gbc.gridy=1;
            gbc.weighty=1;
            gbc.gridx=GridBagConstraints.RELATIVE;
            gbc.gridheight=GridBagConstraints.REMAINDER;
            gbc.gridwidth=1;
        }
        else
        {
            gbc.gridx=1;
            gbc.weightx=1;
            gbc.gridy=GridBagConstraints.RELATIVE;
            gbc.gridwidth=GridBagConstraints.REMAINDER;
            gbc.gridheight=1;
        }
    }
}
