/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
package parka;

import java.awt.*;

/**********************************************************************
  toolbar
  **********************************************************************/

public class ToolBar extends java.awt.Panel
{
    private QueryWin  parent;
    private Tools     tools;
    private TextField text;
    private Checkbox  isvariable;
    private DrawArea  drawArea;

    /*******
     Methods
     *******/

    public ToolBar (QueryWin MyWin)
    {
        parent = MyWin;
        drawArea = null;

        // Create toolbar components
        tools = new Tools (MyWin);
        text = new TextField();
        setTextDisable();
        isvariable = new Checkbox("Variable");
        isvariable.disable();

        setLayout (new BorderLayout(10,0));
        add ("West", tools);
        add ("Center", text);
        add ("East", isvariable);
    }

    void  setDrawArea (DrawArea  da)
    {
	drawArea = da;
    }

    void  resetCursor()
    {
	tools.resetCursor();
    }

    int  getCurrentTool()
    {
	return  tools.getCurrentTool();
    }

    // Modify string if it contains any invalid characters
    public String validateStr(String val)
    {
        return val.replace('#', '*');
    }

    //has to have selected all of text in TextField to return true.
    boolean textIsSelected()
    {
	return  ((text.getSelectionStart() == 0) &&
                  ((text.getSelectionEnd() +1) == (text.getText()).length()) );
    }

    String getTextContents()
    {
	return   (text.getText());
    }

    // event handler for return key (for text field dump)
    public boolean action (Event evt, Object arg)
    {
        if (evt.target instanceof TextField)
        {
            // Test to see if the value is valid
            String val = this.validateStr(text.getText());
            text.setText(val);
            
            if (val.length() > 0) 
            {
		if (drawArea != null)
		{
                    drawArea.changeText(val);
                    drawArea.requestFocus();
		}
            }
            return true;
        }
        // event handler for checkbox
        else if (evt.target instanceof Checkbox)
        {
            if (evt.id == Event.ACTION_EVENT)
                drawArea.toggleVarStatus (isvariable.getState());
            return true;
        }
        
        return false;
    }

    void activateTextField()
    {
	// prepare for changing active Icon's name
        selectText();   //highlights text
	//send focus to TextField
        requestTextFocus();
    }

    void deactivateIconDisplays()
    {
        setText ("");
        setTextDisable();
        setVariableState (false);
        setVariableDisable();
    }

    void activateIconDisplays (String iconName, boolean varState)
    {
        setText (iconName);
        setTextEnable();
        setVariableState (varState);
        setVariableEnable();
    }

    void setVariableState (boolean  newState)
    {
	isvariable.setState (newState);
    }

    void setVariableEnable()
    {
	isvariable.enable();
    }

    void setVariableDisable()
    {
	isvariable.disable();
    }

    private void setTextDisable()
    {
	text.setEditable(false);
    }

    private void setTextEnable()
    {
	text.setEditable(true);
    }

    void setText (String str)
    {
	text.setText (str);
    }

    void requestTextFocus()
    {
	text.requestFocus();
    }

    void selectText()
    {
	text.selectAll();
    }

    public void displayMessage(String msg)
    {
        // Displays a message in the text area
        text.setText(msg);
    }
}
