/* Copyright (C) 1996-1999 PLUS Group - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
   */
/**********************************************************************
  Tools class
  **********************************************************************/
package parka;

import java.awt.*;
import java.awt.image.*;
import java.io.IOException;

class Tools extends java.awt.Canvas
{
    private QueryWin     parent;
    private java.net.URL base;
    private Image        palette;

    private  int tool;

    final static int CREATE = 0;
    final static int MOVE = 1;
    final static int LINK = 2;

    final static String TOOL_PICT_PATH = "palette.gif";

/* no longer one dir too deep:
    final static String BASEPATH_ADJUSTMENT = "../";
*/
    final static String BASEPATH_ADJUSTMENT = "";

    Tools (QueryWin MyWin)
    {
        parent = MyWin;
        tool = MOVE; // init tool

        // if this is an applet:
        if (ApplType.getIsApplet())
        {
            base = ((Parka)parent.parent).getCodeBase();
/* TEST
            System.out.println("in Tools constructor:\n");
            System.out.println("graphics for tool types are in: " + base
                         + TOOL_PICT_PATH);
*/
            palette = ((Parka)parent.parent).getImage (base,
                                BASEPATH_ADJUSTMENT + TOOL_PICT_PATH);
        }
        // if this is an application:
        else  if (ApplType.getIsApplication())
        {
            //'base' includes the filename, in this case-- unlike with the
            //applet.
            base = ((ParkaApp)parent.parent).get_url_for_icons (
                                        TOOL_PICT_PATH);
            ImageProducer  ip;
            try  {
                ip = (ImageProducer)base.getContent();
            }
            catch (IOException io_e)    {
            System.out.println (
                        "Failed to get icons for Query Creation Actions");
            ip = null;
            }
            if (ip != null)
                palette = createImage (ip);
        }
        else
        {
            base = null;
            palette = null;
        }
        resize(65, 32);
    }

    void changeTool (int newTool)
    {
        tool = newTool;
        switch (tool)
        {
            case Tools.CREATE:
                    parent.setCursor(Frame.HAND_CURSOR);
                    break;
            case Tools.MOVE:
                    parent.setCursor(Frame.DEFAULT_CURSOR);
                    break;
            case Tools.LINK:
                    parent.setCursor(Frame.CROSSHAIR_CURSOR);
	}
        repaint();
    }

    void resetCursor()
    {
        switch (tool)
        {
            case Tools.CREATE:
                    parent.setCursor(Frame.HAND_CURSOR);
                    break;
            case Tools.MOVE:
                    parent.setCursor(Frame.DEFAULT_CURSOR);
                    break;
            case Tools.LINK:
                    parent.setCursor(Frame.CROSSHAIR_CURSOR);
	}
        repaint();
    }

    public int getCurrentTool()
    {
	return  tool;
    }

    // override method in class Canvas
    public void paint (Graphics g)
    {
        // System.out.println("in Tools paint");
        if (g == null)
            return;

        super.paint(g);
        g.setPaintMode();

        if (prepareImage(palette, this))
        {
            g.drawImage (palette, 5, 5, this);
            g.setXORMode (Color.white);
            switch (tool)
            {
              case MOVE:
                  g.fillRect(6, 6, 17,19);
                  break;
              case CREATE:
                  g.fillRect(24, 6, 21, 19);
                  break;
              case LINK:
                  g.fillRect(45, 6, 18, 19);
                  break;
            }
        }
    }

    public boolean mouseDown(Event evt, int x, int y)
    {
        // error checking
        //System.out.println("X: " + x + "  Y: " + y);
        if (y > 5 && y < 25)
        {
            if (x > 5 && x < 23)
                tool = MOVE;
            else if (x > 23 && x < 44)
                tool = CREATE;
            else if (x > 44 && x < 63)
                tool = LINK;

	    changeTool (tool);    //go to tool just clicked on.
        }

        // error checking
        //System.out.println("Tool: " + tool);

        paint (this.getGraphics());
        return true;
    }
}
